###########
# initialize methods for models
###########

setMethod("initialize", "dfModel",
  function(.Object, ...)
  {
    modelData<-list(...)$modelData
    modelName<-list(...)$modelName
    modelVar<-list(...)$modelVar
    if (is.null(modelVar))
      modelVar<-list()

    linkData<-list()
    virtualData<-NULL

    # need to check the class of the model data
    if (is(modelData, "matrix"))
      modelData<-as.data.frame(modelData)

    if (is.null(virtualData))
    {
      # need to create the virtualData slot 
      defaultPlotData<-get("defaultPlotData", mvcEnv)
      numRows<-nrow(modelData)
      if (length(defaultPlotData) > 0)
      {
        virData<-data.frame(rep(defaultPlotData[[1]], numRows), 
                            row.names=rownames(modelData))
        virData[,1]<-as.character(virData[,1])
        colnames(virData)<-names(defaultPlotData)[1]
      }
      if (length(defaultPlotData) > 1)
      {
        for (i in 2:length(defaultPlotData))
        {
          virData[,i]<-rep(defaultPlotData[[i]], numRows)
        }
        colnames(virData)<-names(defaultPlotData)
      }
    }
    else
      virData<-virtualData
    # now assign values to the object and then return object
    .Object@modelData<-modelData
    .Object@linkData<-linkData
    .Object@virtualData<-virData
    .Object@modelName<-modelName
    .Object@modelVar<-modelVar
    .Object
  }
)

############
# update model methods
############

setMethod("updateModel", "dfModel",
  function(object, type, data)
  {
    # for a dataframe want to change the virtualData slot
    virData<-virtualData(object)
    dataName<-modelName(object)

    if (type=="reset")
    {
      # then need to reset the virtualData slot to its original values
      # which are stored in defaultPlotData in mvcEnv
      defaultPlotData<-get("defaultPlotData", mvcEnv)
      numRows<-nrow(modelData(object))
      if (length(defaultPlotData) > 0)
      {
        virData<-data.frame(rep(defaultPlotData[[1]], numRows), 
                            row.names=rownames(modelData(object)))
        virData[,1]<-as.character(virData[,1])
        colnames(virData)<-names(defaultPlotData)[1]
      }
      if (length(defaultPlotData) > 1)
      {
        for (i in 2:length(defaultPlotData))
        {
          virData[,i]<-rep(defaultPlotData[[i]], numRows)
        }
        colnames(virData)<-names(defaultPlotData)
      }
    }
    else
    {
      colIndex<-match(type, colnames(virData))
      rowName<-unique(names(data))
      rowIndex<-match(rowName, rownames(virData))

      oldValue<-virData[rowIndex, colIndex]
      # update the value
      virData[rowIndex, colIndex]<-unlist(data)
    }

    # now update the objects
    virtualData(object)<-virData

    # need to update the MVC object - may not be the active MVC
    curMVC<-getMVC(dataName)
    model(curMVC)<-object

    # will also need to update the MVCList
    mvcList <- get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName, allNames)
    mvcList[[index]]<-curMVC
    assign("MVCList", mvcList, mvcEnv)

    # return the data needed to update the views
    # need the old value to remove the old point from the plot before
    # re-adding the point with the new value
    if (type=="reset")
      viewData<-list()
    else
      viewData<-list(colName=type, rowName=rowName, oldValue=oldValue, 
                   newValue=unlist(data))
    return(viewData)
  }
)

##############
# redraw view methods
##############

setMethod("redrawView", "sPlotView",
  function(object)
  {
    viewItem<-scatterplot(object)
    curDevice<-plotDevice(viewItem)

    # reassign this view to the MVC
    curMVC<-getMVC(dataName(object))
    curVList<-viewList(curMVC)
    # now find where this view fits in the viewList
    booSplot<-unlist(lapply(curVList, function(x) 
                     {
                       if (is(x, "plotView"))
                         return(curDevice==plotDevice(x))
                       else
                         return(FALSE)
                     }))
    splotIndex<-which(booSplot)
    # now reassign the spreadsheet
    curVList[[splotIndex]]<-viewItem
    viewList(curMVC)<-curVList

    # now reassign to MVCList
    MVCList<-get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName(object), allNames)
    MVCList[[index]]<-curMVC
    assign("MVCList", MVCList, mvcEnv)    
  }
)

setMethod("redrawView", "spreadView",
  function(object)
  {
    curMVC<-getMVC(dataName(object))
    curWin<-win(object)

    curCList<-clist(object)

    # actually get the data
    showdata<-getData(name=dataName(object))    

    # first freeze the view before changing it
    curCList$Freeze()

    # need to remove all rows and then put them back in
    clistLength<-gtkCListGetRows(curCList)
    for (i in 1:clistLength)
    {
      # always remove the first row
      gtkCListRemove(curCList, 0)
    }

    # now need to readd the rows
    for (i in 1:nrow(showdata))
    {
      text<-c(row.names(showdata)[i], as.matrix(showdata[i,]))
      curCList$Append(text)
    }
    curCList$Thaw()

    # reassign this view to the MVC
    curVList<-viewList(curMVC)
    # now find where this view fits in the viewList
    booSpread<-unlist(lapply(curVList, function(x) {
                                          is(x, "spreadView")
                                        }))
    spreadIndex<-which(booSpread)
    # now reassign the spreadsheet
    curVList[[spreadIndex]]<-object
    viewList(curMVC)<-curVList

    # now reassign to MVCList
    MVCList<-get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName(object), allNames)
    MVCList[[index]]<-curMVC
    assign("MVCList", MVCList, mvcEnv)
  }
)

###########
# update view methods
###########

setMethod("updateView", "sPlotView",
  function(object, vData)
  {
    # vData is a list with 4 elements: rowName, colName, oldValue and newValue
    # will need to remove old value and draw point with new value

    # get the model info.
    dataName<-dataName(object)
    curMVC<-getMVC(dataName)
    model<-model(curMVC)
    modelData<-modelData(model)
    virtualData<-virtualData(model)

    # set the current device
    curDevice<-plotDevice(object)
    dev.set(curDevice)

    # will need the background color
    bgColor<-plotPar(object)$bg
    if (bgColor=="transparent")
      bgColor<-"white"

    rowName<-vData$rowName
    # this will be a column in the virtualData slot
    colName<-vData$colName
    oldValue<-vData$oldValue
    newValue<-vData$newValue

    # these are columns in the modelData slot
    varx<-xvar(object)
    vary<-yvar(object)
    dfRows<-dfRows(object)
    # only update this point if it is being shown in the current plot
    if (all(rowName %in% dfRows))
    {
      # get the x and y values
      rowIndex<-match(rowName, rownames(modelData))
      xcolIndex<-match(varx, colnames(modelData))
      ycolIndex<-match(vary, colnames(modelData))
      xvalue<-modelData[rowIndex, xcolIndex]
      yvalue<-modelData[rowIndex, ycolIndex]

      pchIndex<-match("pch", colnames(virtualData))
      pchValue<-virtualData[rowIndex, pchIndex]
      colIndex<-match("color", colnames(virtualData))
      colValue<-virtualData[rowIndex, colIndex]
      highIndex<-match("highlit", colnames(virtualData))
      highValue<-virtualData[rowIndex, highIndex]

      # next remove the old point
      # what about highlighting??? - would also need to remove this!
      if (colName != "pch")
        points(xvalue, yvalue, col=bgColor, pch=as.numeric(pchValue))
      else
        points(xvalue, yvalue, col=bgColor, pch=as.numeric(oldValue))

      # remove highlighting if it's needed
      if (all(highValue==TRUE) && colName!="highlit")
        points(xvalue, yvalue, col=bgColor, cex=2, pch=1)
      if (colName=="highlit" && all(newValue==FALSE))
        points(xvalue, yvalue, col=bgColor, cex=2, pch=1)

      # only redraw the point if it's not supposed to be hidden
      if (!(colName=="hide" && all(newValue==TRUE))) 
      {
        # then redraw the point
        points(xvalue, yvalue, col=colValue, pch=as.numeric(pchValue))

        # add highlighting if it's needed - always use a circle for 
        # highlighting so it's consistent (and not dependent on the current
        # pch value)
        if (all(highValue==TRUE))
          points(xvalue, yvalue, col="red", pch=1, cex=2)
      }
    }
  }
)

setMethod("updateView", "spreadView",
  function(object, vData)
  {
    # vData is a list with 4 elements: rowName, colName, oldValue and newValue

    # get the model info. - may not be from the active MVC
    curMVC<-getMVC(dataName(object))
    model<-model(curMVC)
    modelData<-modelData(model)
    virtualData<-virtualData(model)

    curCList<-clist(object)

    colName<-vData$colName
    rowName<-vData$rowName
    rowIndex<-match(rowName, rownames(modelData))
    
    if (colName=="color" || colName=="pch")
    {
      # always select the row when a point is being colored
      gtkCListSelectRow(curCList, rowIndex-1, -1)
    }
    else
    {
      # for highlight and hide need to toggle the selection
      # if value==TRUE then select, otherwise unselect
      newValue<-vData$newValue
      if (newValue==TRUE)
        gtkCListSelectRow(curCList, rowIndex-1, -1)
      else
      {
        gtkCListUnselectRow(curCList, rowIndex-1, -1)
      }
    }    
  }
)

##############
# click event and motion event methods for views
##############

########
# clickEvent for spreadView is called in response to a selected row
# or unselected row from the clist on a spreadView
########
setMethod("clickEvent", "spreadView", 
  function(object, where, ...)
  {
    extras<-list(...)
    # event is select or unselect
    event<-extras$event

    # assume that the clickEvent for spreadView has the same callback
    # function as the left button click because a row can only be selected
    # or unselected with the left button

    # may want to check if the view clicked on is from the activeMVC
    # 5/30/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)
      modelData<-modelData(model(curMVC))
      rowName<-rownames(modelData)[where]

      # see if there is a function to call
      leftButtonClick<-get("leftButtonClick", controlEnv)

      # see if any of the functions are active
      activeLBCFun<-unlist(lapply(leftButtonClick, function(y)
        {y$assigned==TRUE}))

      curIndex<-which(activeLBCFun)
      if (length(curIndex)==1)
      { 
        curEventFun<-leftButtonClick[[curIndex]][["eventFun"]]

        actualFun<-callFun(curEventFun)
        # problem is that we can't list it as select or unselect event
        # but that's ok because highlight and hide will be toggled anyway
        do.call(actualFun, list(rowName))
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

#########
# create a button press event for scatterplot views (used to be
# addButtonPress function)
#########
setMethod("clickEvent", "sPlotView",
  function(object, where, ...)
  {
    # may want to check if the view clicked on is from the activeMVC
    # 5/30/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)

      # see if there is a function to call
      rightButtonClick<-get("rightButtonClick", controlEnv)
      leftButtonClick<-get("leftButtonClick", controlEnv)
      middleButtonClick<-get("middleButtonClick", controlEnv)

      # see if any of the functions are active
      activeRBCFun<-unlist(lapply(rightButtonClick, function(y)
        {y$assigned==TRUE}))
      activeLBCFun<-unlist(lapply(leftButtonClick, function(y)
        {y$assigned==TRUE}))
      activeMBCFun<-unlist(lapply(middleButtonClick, function(y)
        {y$assigned==TRUE}))

      # where will be the mouse click event information from Gtk
      # check that the left button was pressed
      if (gdkEventButtonGetButton(where)==1)
      {
        if (any(activeLBCFun))
        {
          usri <- pix2inches(where[["X"]], where[["Y"]])
          usrc <- inches2usr(usri$x, usri$y)
          
          curPoint<-identifyPoint(object, usrc)
          
          if (!is.null(curPoint))
          {
            # if a point was clicked, then we want to create a 
            # gUpdateDataMessage this is done in the active eventFun function

            curIndex<-which(activeLBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-leftButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # point as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curPoint$closestPoint))
            }
          }
        }
      }

      if (gdkEventButtonGetButton(where)==2)
      {
        if (any(activeMBCFun))
        {
          usri <- pix2inches(where[["X"]], where[["Y"]])
          usrc <- inches2usr(usri$x, usri$y)

          curPoint<-identifyPoint(object, usrc)

          if (!is.null(curPoint))
          {
            curIndex<-which(activeMBCFun)
            if (length(curIndex)==1)
            { 
              curEventFun<-middleButtonClick[[curIndex]][["eventFun"]]

              # now call the function the user has set with the current
              # point as the parameter 
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curPoint$closestPoint))
            }
          }
        }
      }

      if (gdkEventButtonGetButton(where)==3)
      {
        if (any(activeRBCFun))
        {
          usri <- pix2inches(where[["X"]], where[["Y"]])
          usrc <- inches2usr(usri$x, usri$y)

          curPoint<-identifyPoint(object, usrc)

          if (!is.null(curPoint))
          {
            curIndex<-which(activeRBCFun)
            if (length(curIndex)==1)
            {
              curEventFun<-rightButtonClick[[curIndex]][["eventFun"]]
  
              # now call the function the user has set with the current
              # point as the parameter
              actualFun<-callFun(curEventFun)
              do.call(actualFun, list(curPoint$closestPoint))
            }
          }
        }
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

########
# a method for a motion notify event on a scatterplot
########
setMethod("motionEvent", "sPlotView",
  function(object, event, ...)
  {
    # may want to check if the view clicked on is from the activeMVC
    # 5/30/06 respond to any events (even if the view does not belong to the
    # active MVC)
    activeMVC<-get("activeMVC", mvcEnv)
#    if (dataName(object) == activeMVC)
#    {
      curMVC<-getMVC(dataName(object))
      controlEnv<-controller(curMVC)

      # see if there is a function to call
      mouseOver<-get("mouseOver", controlEnv)
      # see if any of the functions are active
      activeFun<-unlist(lapply(mouseOver, function(y){y$assigned==TRUE}))

      # then there is an active function
      if (any(activeFun))
      {
        curWin<-win(object)
        # potential problem - these depend on how the 
        # scatterplot window is set up
        # SHOULD AT LEAST CHECK THEIR CLASSES!!
        hrul<-gtkContainerGetChildren(
             gtkContainerGetChildren(curWin)[[1]])[[1]]
        vrul<-gtkContainerGetChildren(gtkContainerGetChildren(
             gtkContainerGetChildren(curWin)[[1]])[[2]])[[1]]

        hrul$SignalEmit("motion-notify-event", event)
        vrul$SignalEmit("motion-notify-event", event)
      
        curx<-gdkEventMotionGetX(event)
        cury<-gdkEventMotionGetY(event)

        # convert to user coordinates
        xyInches<-pix2inches(curx, cury)
        xyUsr<-inches2usr(xyInches$x, xyInches$y)
 
        # also need to ensure that the plot has been added to viewList
        curDev<-dev.cur()
        curViewList<-viewList(curMVC)

        devicesInView<-unlist(lapply(curViewList, function(x) {is(x,
                             "plotView")}))
        # only look at views that have devices
        viewsWithDevices<-curViewList[devicesInView]
        plotDev<-unlist(lapply(viewsWithDevices, function(x) {plotDevice(x)}))

        if (curDev %in% plotDev)
        {
          curPoint<-identifyPoint(object, xyUsr)

          curIndex<-which(activeFun)
          if (length(curIndex)==1)
          {
            curEventFun<-mouseOver[[curIndex]][["eventFun"]]

            # now call the function the user has set with the current
            # point as the parameter - assume the function does not return
            # anything to the motion notify event
            actualFun<-callFun(curEventFun)
            do.call(actualFun, list(curPoint$closestPoint))
          }
        }
      }
#    }
    # return TRUE if there are no errors because gtkAddCallback is expecting
    # a boolean
    return(TRUE)
  }
)

############
# initialize methods for views
############

# dataName, dfRows, xvar and yvar are the only data needed 
# to create scatterplot
setMethod("initialize", "sPlotView", 
  function(.Object, ...)
  {
    dataName<-list(...)$dataName
    mData<-list(...)$mData

    dfRows<-mData$dfRows
    xvar<-mData$xvar
    yvar<-mData$yvar

    # create slot values
    retList<-initSPlotView()

    # fill the slots in the object
    .Object@dataName<-dataName
    .Object@win<-retList$win
    .Object@plotDevice<-retList$plotDevice
    .Object@plotPar<-retList$plotPar
    .Object@drArea<-retList$drArea
    .Object@dfRows<-dfRows
    .Object@xvar<-xvar
    .Object@yvar<-yvar

    # create the actual plot
    .Object<-scatterplot(.Object)

    # finally add events to the plot
    .Object<-addEventsforViews(.Object)
    addEventsforSPlots(.Object)

    .Object
  }
)

setMethod("initialize", "spreadView", 
  function(.Object, ...)
  {
    dataName<-list(...)$dataName

    # create the slot values
    retList<-initSpreadView(dataName)

    # fill the slots in the object
    .Object@dataName<-dataName
    .Object@win<-retList$win
    .Object@clist<-retList$clist

    # finally add events to the view
    .Object<-addEventsforViews(.Object)
    addEventsforSpread(.Object)

    .Object
  }
)

############
# initialize methods for messages
############

setMethod("initialize", "gUpdateViewMessage", 
  function(.Object, ...)
  {
    .Object@type<-list(...)$type
    .Object@mData<-list(...)$mData
    dataName<-list(...)$dataName
    if (dataName == "")
      dataName<-get("activeMVC", mvcEnv)
    .Object@dataName<-dataName

    .Object
  }
)

#####
# update the data stored in the modelData slot of a model
#####
setMethod("initialize", "gUpdateDataMessage", 
  function(.Object, ...)
  {
    .Object@type<-list(...)$type
    .Object@mData<-list(...)$mData
    dataName<-list(...)$dataName
    from<-list(...)$from
    if (dataName == "")
      dataName<-get("activeMVC", mvcEnv)
    if (from == "")
      from<-dataName
    .Object@dataName<-dataName
    .Object@from<-from

    .Object
  }
)

#######
# data must be a list!!!
# data may contain only the model data or it may contain the model data AND
# the link data
#######
setMethod("initialize", "gAddDataMessage",
  function(.Object, ...)
  {
    .Object@dataName<-list(...)$dataName
    .Object@mData<-list(...)$data
    .Object@type<-"data.frame"
    
    .Object
  }
)

setMethod("initialize", "gAddViewMessage",
  function(.Object, ...)
  {
    .Object@dataName<-list(...)$dataName
    .Object@type<-list(...)$type
    .Object@mData<-list(...)$mData

    .Object
  }
)

##############
# handle message methods for messages
##############

setMethod("handleMessage", "gUpdateViewMessage",
  function(object, ...)
  {
    type<-type(object)
    viewdata<-mData(object)
    dataName<-dataName(object)

    # now need to update the view - note it may not be the active MVC
    curMVC<-getMVC(dataName)
    curVList<-viewList(curMVC)
    controlEnv<-controller(curMVC)

    if (length(dev.list()) > 0)
      activeDev<-dev.cur()

    assign("contLoop", TRUE, controlEnv)
    controller(curMVC)<-controlEnv
    MVCList<-get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName, allNames)
    MVCList[[index]]<-curMVC
    assign("MVCList", MVCList, mvcEnv)

    if (type=="updateView")
    {
      for (i in 1:length(curVList))
      {
        updateView(curVList[[i]], viewdata) 
      }
    }
    if (type=="redrawView")
    {
      for (i in 1:length(curVList))
      {
        # for type of "redrawView", mData will be an empty list
        redrawView(curVList[[i]])
      }
    }

    # need to get this again because the MVC object has changed
    curMVC<-getMVC(dataName)
    curVList<-viewList(curMVC)
    controlEnv<-controller(curMVC)

    assign("contLoop", FALSE, controlEnv)
    controller(curMVC)<-controlEnv
    MVCList<-get("MVCList", mvcEnv)
    allNames <- getModelNames(sort = FALSE)

    index<-match(dataName, allNames)
    MVCList[[index]]<-curMVC
    assign("MVCList", MVCList, mvcEnv)

    # reset the active device
    if (length(dev.list()) > 0)
      dev.set(activeDev)
  }
)

########
# added on 7/14/05 so that we also update parent/children of the model
########
setMethod("handleMessage", "gUpdateDataMessage",
  function(object, ...)
  {
    data<-mData(object)
    type<-type(object)

    dataName<-dataName(object)
    from<-from(object)

    # now need to update the model
    curMVC<-getMVC(dataName)
    curModel<-model(curMVC)
    # will get an error here if print statement is used when the model to
    # be updated is a graph

    # a method for each model to update its data
    viewdata<-updateModel(curModel, type, data)

    # update the views now that the data has changed
    # check that the model has views also!
    curVList<-viewList(curMVC)
    if (length(curVList) > 0)
    {
      if (type=="reset")
      {
        uvMessage<-new("gUpdateViewMessage", type="redrawView", mData=viewdata,
                       dataName=dataName)
        handleMessage(uvMessage) 
      } 
      else
      {
        # create an update view message
        # for now assume that all update view messages that come from update
        # data will have type "updateView", rather than "redrawView"
        uvMessage<-new("gUpdateViewMessage", type="updateView", mData=viewdata,
                      dataName=dataName)
#        print(uvMessage)
        handleMessage(uvMessage)
      }
    }
  }
)

########
# 6/1/05
# load models through messages
########
setMethod("handleMessage", "gAddDataMessage",
  function(object, ...)
  {
    data<-mData(object)$data
    name<-dataName(object)

    loadModel(data, name)
  }
)

#######
# 5/24/05 want to initialize a view through methods rather than a function
#######
setMethod("handleMessage", "gAddViewMessage",
  function(object, ...)
  {
    dataName<-dataName(object)
    type<-type(object)
    mData<-mData(object)

    if (length(mData) > 0)
      newView<-new(type, dataName=dataName, mData=mData)
    else
      newView<-new(type, dataName=dataName)
  }
)



