##########
# new functions for these new classes
##########

#######
# called by many functions in MVCClass and iSNetwork
#######
getMVC<-function(mName)
{
  mvcList<-get("MVCList", mvcEnv)
  allNames<-getModelNames(sort=FALSE)
  index<-match(mName, allNames)
  curMVC<-mvcList[[index]]
  return(curMVC)
}

#########
# called by many functions in MVCClass and iSNetwork
#########
getModelNames<-function(sort=TRUE)
{
  mvcList<-get("MVCList", mvcEnv)
  if (length(mvcList) > 0)
  {
    allNames<-unlist(lapply(lapply(mvcList, model), modelName))
    # return the names in alphabetical order
    if (sort)
      return(sort(allNames))
    else
      return(allNames)
  }
  else
    return(NULL)
}

#######
# load the data into the mvcEnv
# data is the name of data variable
# type determines the model class
# name is the character string name of the model
# linkData is the link data
#
# called by handleMessage method for gAddDataMessage
#######
loadModel<-function(data, name)
{
  booLoad<-TRUE
  # should check that the model name is not already in use!!!
  allMNames<-getModelNames()
  if (length(allMNames) > 0)
  {
    if (name %in% allMNames)
    {
      booLoad<-FALSE
      if (length(get("controlWin", mvcEnv)) > 0)
        showLabel("That model name is already in use.")
      else
        print("That model name is already in use.")
    }
  }

  if (booLoad)
  {
    # first create the appropriate model object
    newObject<-new("dfModel", modelData=data, modelName=name, 
                      linkData=list())

    # now need to create a MVC object and add to MVCList in mvcEnv
    # leave parentMVC slot as NULL
    newMVC<-new("singleModelMVC", model=newObject, viewList=list(), 
				  controller=new.env())

    # need to add the default values to the new controller environment
    setControllerDefaults(newMVC)
    createDefaultCallEvents(newMVC)
 
    MVCList<-get("MVCList", mvcEnv)
    if (length(MVCList) > 0)
    { 
      MVCList[[length(MVCList)+1]]<-newMVC
      assign("MVCList", MVCList, mvcEnv)
    }
    else
    {
      assign("MVCList", list(newMVC), mvcEnv)
      # if this is the first model, then it will be the active MVC
      assign("activeMVC", modelName(model(newMVC)), mvcEnv)
    }
  }
}

############
# show a label in a Gtk window
#
# called by loadModel function
############
showLabel<-function(lblText)
{
  win<-gtkWindow(show=FALSE)
  hbox<-gtkHBox()
  win$Add(hbox)
  lbl<-gtkLabel(lblText)
  gtkBoxPackStart(hbox, lbl, padding=5)
  okBut<-gtkButton("Ok")
  okBut$SetUsize(40, 20)
  gtkBoxPackStart(hbox, okBut, padding=5)
  
  gtkAddCallback(okBut, "clicked",
    function(obj)
    {
      win$Destroy()
      return(TRUE)
    }
  )
  win$Show()
}

#########
# 5/24/05 create a scatterplot view through initialize method
# here just setting up the device
#
# called by initialize method for heatmapView and sPlotView
#########
initSPlotView<-function()
{
  win<-gtkWindow(show=FALSE)
  
  # this will create a gtk device on the window
  retList<-createGtkDev(win)
  win<-retList$win
  drArea<-retList$drArea

  newRetList<-list(win=win, drArea=drArea, plotDevice=dev.cur(),
                   plotPar=par(no.readonly=TRUE))

  return(newRetList)
}

############
# 5/24/05 add events for any type of view
# this also adds the view to the viewlist and updates
# the mvclist
#
# called by initialize method for all views
############
addEventsforViews<-function(newView)
{
  dataName<-dataName(newView)

  MVCList<-get("MVCList", mvcEnv)
  allNames<-getModelNames(sort=FALSE)
  mvcindex<-match(dataName, allNames)
  curMVC<-getMVC(dataName)
  controlEnv<-controller(curMVC)

  win<-win(newView)

  assign("activeView", newView, controlEnv)

  # need to add a callback for focus_in_event
  gtkAddCallback(win, "focus_in_event",
    function(obj, ev)
    {
      if (is(newView, "plotView"))
      {
        # set the active device
        dev.set(plotDevice(newView))
      }
      # set the active view
      assign("activeView", newView, controlEnv)
      # 6/1/06 need to store the active window number so that the view with
      # focus can be determined (this is a problem if the view doesn't have
      # a device)
      assign("activeWinNum", winNum(newView), mvcEnv)

      return(TRUE)
    }
  )

  # need a callback for focus_out_event to remove any open tooltips
  gtkAddCallback(win, "focus_out_event",
    function(obj, ev)
    {
      # no active view
      assign("activeView", c(), controlEnv)
      assign("activeWinNum", c(), mvcEnv)
     
      return(TRUE)
    }
  )

  # add the key press events for pseudo-accelerators
  createKeyPressEvent(win)        

  # add the delete event
  closeWin(newView)

  # add a new sub menu item to the control window
  winCounter<-get("winCounter", mvcEnv)
  assign("winCounter", winCounter+1, mvcEnv)

  curLabel<-paste("Window _", get("winCounter", mvcEnv),
                  "  Ctrl+", get("winCounter", mvcEnv), sep="")
  if (length(get("controlWin", mvcEnv)) > 0)
  {
    newMenuItem<-addSubMenuItem("window", curLabel, "activateWindow",  
                 winNum=get("winCounter", mvcEnv))
  }
  else
  {
    addToKeyVals(text=curLabel, action="activateWindow", 
                   winNum=get("winCounter", mvcEnv))
  }
  winNum(newView)<-get("winCounter", mvcEnv)

  # set the title of the window
  curTitle<-paste("Window ", get("winCounter", mvcEnv), sep="")
  gtkWindowSetTitle(win, curTitle)

  # add new object to viewList      
  curviewList<-viewList(curMVC)
  viewLen<-length(curviewList)
  curviewList[[viewLen+1]]<-newView
  viewList(curMVC)<-curviewList

  # need to reassign the curMVC to MVCList
  MVCList[[mvcindex]]<-curMVC
  assign("MVCList", MVCList, mvcEnv)

  return(newView)
}

##########
# 5/24/05 add events specifically for scatterplots
#
# called by initialize method for sPlotView
##########
addEventsforSPlots<-function(curView)
{
  gtkAddCallback(drArea(curView), "button_press_event",
    function(obj, ev)
    {
      clickEvent(curView, ev)
      return(TRUE)
    }
  )
      
  gtkAddCallback(drArea(curView), "motion-notify-event",
    function(obj, ev)
    {
      motionEvent(curView, ev)
      return(TRUE)
    }
  )  
}

#########
# 5/24/05 create a spreadsheet view through initialize method
#
# called by initialize method for spreadView
#########
initSpreadView<-function(dfName, win=c())
{
  if (length(win)==0)
    win<-gtkWindow(show=FALSE)

  win$SetUsize(500, 400)
  newTable<-gtkTable(1, 1, TRUE)

  # actually get the data
  showdata<-getData(name=dfName)
  # need to add one to include the row names as a column
  numCols<-(dim(showdata)[2])+1
  numRows<-dim(showdata)[1]
  # need to add a frame that shows the data
  frData<-gtkFrame(dfName)     

  boxForShow<-gtkHBox(TRUE, 2)
  scShow<-gtkScrolledWindow()
  scShow$SetPolicy("automatic", "automatic")

  showDfList<-gtkCList(numCols, titles=c("Row.Names", names(showdata)))
  showDfList$Freeze()
  for (i in 1:numRows)
  {
    text<-c(row.names(showdata)[i], as.matrix(showdata[i,]))
    showDfList$Append(text)
  }
  showDfList$Thaw()
  # make the column titles just be labels rather than buttons
  gtkCListColumnTitlesPassive(showDfList)
  # also want to set the column width so all items in the column
  # can be seen
  gtkCListColumnsAutosize(showDfList)
  # set the selection mode to multiple so multiple items can be selected
  # at a time (not sure if I want extended)
  gtkCListSetSelectionMode(showDfList, GtkSelectionMode[3])

  scShow$AddWithViewport(showDfList)
  boxForShow$PackStart(scShow, expand=TRUE)
  frData$Add(boxForShow)
  gtkTableAttach(newTable, frData, 0, 1, 0, 1, ypadding=10, xpadding=5)
  win$Add(newTable)
  win$Show()

  newRetList<-list(win=win, clist=showDfList)

  return(newRetList)
}

#########
# 5/24/05 add events specifically for spreadsheets
#
# called by initialize method for spreadView
#########
addEventsforSpread<-function(curView)
{
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)
  controlEnv<-controller(curMVC)

  curclist<-clist(curView)
  gtkAddCallback(curclist, "select-row",
    function(obj, row, col, ev)
    {
      # need this variable to make sure don't end up in an infinite loop
      if (get("contLoop", controlEnv)==FALSE)
        clickEvent(curView, (row+1), event="select")
      return(TRUE)
    }
  )
   
  gtkAddCallback(curclist, "unselect-row",
    function(obj, row, col, ev)
    {
      # need this variable to make sure don't end up in an infinite loop
      if (get("contLoop", controlEnv)==FALSE)
        clickEvent(curView, (row+1), event="unselect")
      return(TRUE)
    }
  )
}

############
# create the list of gEventFun objects that are loaded initially
############
createDefaultCallEvents<-function(newMVC)
{
  # callEvents for the df model must have a callback function that takes
  # a row name as the only parameter
  if (is(model(newMVC), "dfModel"))
  {
    addCallEvent(tooltips, "show tooltips over points", list(setTooltipsGUI), 
                 "mouseOver", newMVC)
    addCallEvent(highPointByMotion, 
                 "highlight points as cursor moves over them", NULL,
                 "mouseOver", newMVC)
    addCallEvent(colorPoint, "color a point", list(createColorBrowser),
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)
    # there is no preprocessing function for hiding a point
    addCallEvent(hidePoint, "hide a point", NULL, 
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)
    addCallEvent(highlightPoint, "highlight a point", NULL,
                 c("leftButtonClick", "middleButtonClick",
                 "rightButtonClick"), newMVC)

    # check to see if the user has added any callback functions through
    # the command line
    newCallFuns<-get("newCallFuns", mvcEnv)
    if (length(newCallFuns) > 0)
    {
      for (i in 1:length(newCallFuns))
      {
        curCallFun<-newCallFuns[[i]]
        addCallEvent(curCallFun$callFunction, curCallFun$shortName, 
                     curCallFun$preprocessingFun, curCallFun$eventLists, 
                     newMVC)
      }
    }
  }
}

#############
# to add a gEventFun object to one of the event lists in controlEnv
#  callFunction is callFun in gEventFun
#  shortName is shortName in gEventFun
#  preprocessingFun is preprocessFun in gEventFun
#  eventLists is a vector of the events that this gEventFun object can 
#   be linked to (each of these should be an environment variable in 
#   controlEnv and should be an element in possEvents in controlEnv)
#  newMVC is the MVC object that this callback function will be added to (in 
#   its controlEnv)
#############
addCallEvent<-function(callFunction, shortName, preprocessingFun, eventLists, 
                       newMVC)
{
  controlEnv<-controller(newMVC)
  if (all(eventLists %in% get("possEvents", mvcEnv)))
  {
    # create new gEventFun object
    newEventFun<-new("gEventFun", callFun=callFunction, shortName=shortName,
                     preprocessFun=preprocessingFun)

    for (i in 1:length(eventLists))
    {
      curEventList<-get(eventLists[i], controlEnv)
    
      # then add to list and reassign
      eventLen<-length(curEventList)
      # when adding a new element to the event list, assume the event is not
      # currently linked to this callback function - that will be assigned
      # later
      curEventList[[eventLen+1]]<-list(eventFun=newEventFun, assigned=FALSE)

      assign(eventLists[i], curEventList, controlEnv)
    }
  }
}

##########
# this function gets the MVC that the view with focus belongs to - does this
# using the winNum slot (which is stored in mvcEnv as activeWinNum)
#
# returns the MVC that corresponds with the view that has focus or it returns
# the active MVC if the MVC corresponding to the view with focus can't be 
# found
##########
getMVCFromWinNum<-function()
{
  aWinNum<-get("activeWinNum", mvcEnv)
  if (length(aWinNum) == 0)
  {
    return(getMVC(get("activeMVC", mvcEnv)))
  }
  else
  {
    allMVCs<-getMVCList()
    # this will be a list of lists
    allViewLists<-lapply(allMVCs, viewList)

    winNumValues<-lapply(allViewLists, function(x)
       { unlist(lapply(x, function(y) 
         {         
           y@winNum
         } ))
       })
    winIndex<-unlist(lapply(winNumValues, function(x) aWinNum %in% x))
    if (any(winIndex))
    {
      curMVC<-allMVCs[[which(winIndex)]]
      return(curMVC)
    }
    else
      return(getMVC(get("activeMVC", mvcEnv)))
  }
}

##########
# this is the function to call to set the tooltips in the control environment
##########
setTooltipsGUI<-function()
{  
  if (length(get("controlWin", mvcEnv)) > 0)
  { 
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)

    if (getNumberofViews() > 0)
    {
      # first create a gtk window that will show the tooltips options
      # need the window to be modal
      assign("notDone", TRUE, controlEnv)

      w<-gtkWindow(show=FALSE)
      vbox<-gtkVBoxNew()
      w$Add(vbox)
      lab<-gtkLabel("Tooltip Options")
      gtkBoxPackStart(vbox, lab)

      curModel<-model(curMVC)

      curData<-modelData(curModel)
      if (ncol(curData) > 0)
        extraTTs<-colnames(curData)
      else
        extraTTs<-c()
  
      # now need to add radio buttons
      radButs<-list()
   
      radBut<-gtkRadioButtonNewWithLabelFromWidget(label="Row Names")
      radButs[[1]]<-radBut
      gtkBoxPackStart(vbox, radBut)
      if (length(extraTTs) > 0)
        for (i in 1:length(extraTTs))
        {
          newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                           label=extraTTs[i])
          gtkBoxPackStart(vbox, newradBut)
          radButs[[i+1]]<-newradBut
        }

      # now add a set button
      but<-gtkButton("Set Tooltips")
      gtkBoxPackStart(vbox, but)    

      gtkAddCallback(but, "clicked",
        function(obj, ev)
        {
          # find out which button is active
          for (i in 1:length(radButs))
          {      
            if (gtkObjectGetArgs(radButs[[i]], "active") == TRUE)
              curIndex<-i
          }

          # get the row names
          rNames<-rownames(curData)
          if (curIndex==1)
          {
            # set the tooltips to the row name
            ttVec<-rNames
            names(ttVec)<-rNames
            assign("ttToShow", ttVec, controlEnv)
          }
          else
          {
            cNames<-colnames(curData)
            ttVec<-curData[,(curIndex-1)]
            names(ttVec)<-rNames
            assign("ttToShow", ttVec, controlEnv)
          }
        
          w$Destroy()
          assign("notDone", FALSE, controlEnv)
          return(TRUE)
        }
      )
      w$Show()
      while (get("notDone", controlEnv))
        Sys.sleep(0.5)
    }
  }
  else
    print(paste("Please call the function, setTooltips,", 
            "to set the tooltip names.  The default tooltips are the",
            "row names from the data set, but the setTooltips function",
            "lets the user set the tooltips to a column from the data frame."))
}

###########
# to show a tooltip over a point
###########
tooltips<-function(curName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/30/06 added so that users can interact with views that don't belong to 
  # the active MVC
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)

  # need to get the named vector which will be shown as tooltips
  # this must be set in a previous function
  ttToShow<-get("ttToShow", controlEnv)

  # if the tooltips have not been set then use the default of the row names
  if (length(ttToShow) == 0)
  {
    curModel<-model(curMVC)
    curData<-modelData(curModel)    
    rNames<-rownames(curData)

    # set the tooltips to the row name
    ttVec<-rNames
    names(ttVec)<-rNames
    ttToShow<-ttVec
  }

  if (!is.null(curName))
  {
    curIndex<-match(curName, names(ttToShow))
    if (any(is.na(curIndex)))
    {
      if (length(curName)==1)
        curToolTip<-curName
      else
        curToolTip<-paste(curName, collapse="\n")
    }
    else
    {
      if (length(curIndex)==1)
        curToolTip<-ttToShow[curIndex]
      else
        curToolTip<-paste(ttToShow[curIndex], collapse="\n")
    }
  }
  else
    curToolTip<-""

  # check if a tooltip is currently being shown
  oldName<-get("oldTTName", controlEnv)
  oldTTWin<-get("oldTTWin", controlEnv)
      
  if (is.null(curName))
  {          
    # no old name and no new name
    if (all(oldName==rep("", length(oldName))))
    {
      # nothing needs to be updated
    }
    else                            # old name was shown, but no new name
    {
      assign("oldTTName", "", controlEnv)
      if (length(oldTTWin) > 0)  # which it should be
      {
        oldTTWin$Destroy()
        assign("oldTTWin", c(), controlEnv)
      }
    }
  }
  else
  {
    # no old name, but a new name to show
    if (all(oldName==rep("", length(oldName))))
    {
      w<-gtkWindow(type=GtkWindowType[3], show=FALSE)
      lab<-gtkLabel(curToolTip)
      w$Add(lab)
      gtkWindowSetPosition(w, position=GtkWindowPosition[3])
      w$Show()

      assign("oldTTName", curName, controlEnv)
      assign("oldTTWin", w, controlEnv)          
    }
    else
    {
      if (all(oldName==curName))
      # old name and new name are the same
      {
        # nothing needs to be updated
      }
      else                          # old name is different from new name
      {
        # destroy old tooltip
        oldTTWin$Destroy()
            
        w<-gtkWindow(type=GtkWindowType[3], show=FALSE)
        lab<-gtkLabel(curToolTip)
        w$Add(lab)
        gtkWindowSetPosition(w, position=GtkWindowPosition[3])
        w$Show()

        assign("oldTTName", curName, controlEnv)
        assign("oldTTWin", w, controlEnv)                 
      }
    }
  }
}

##########
# 6/7/05 added
# highlight points as the cursor moves over them
##########
highPointByMotion<-function(rowName)
{
  # this is the model name that is active
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/30/06 added so that users can interact with views that don't belong to 
  # the active MVC
  curMVC<-getMVCFromWinNum()
  mName<-modelName(model(curMVC))
  controlEnv<-controller(curMVC)
  model<-model(curMVC)
  virtualData<-virtualData(model)  

  oldMotionRow<-get("oldMotionRow", controlEnv)

  if (length(rowName) == 0)
    rowName<-""

  # note that if the 2 variables don't have the same length, then they
  # are not the same
  booSame<-TRUE
  if (length(rowName) != length(oldMotionRow))
    booSame<-FALSE
  else
    if (all(rowName != oldMotionRow))
      booSame<-FALSE

  if (!booSame)
  {
    colName<-"highlit"
    # remove old highlighting
    testOldEmpty<-rep("", length(oldMotionRow))
    testNewEmpty<-rep("", length(rowName))
    if (all(oldMotionRow != testOldEmpty))
    {
      # then need to toggle old highlighting
      highIndex<-match(colName, colnames(virtualData))
      rowIndex<-match(oldMotionRow, rownames(virtualData))
      highValue<-virtualData[rowIndex, highIndex]
      # toggle the highlighting
      newHValue<-!highValue

      mData<-as.list(newHValue)
      names(mData)<-oldMotionRow

      # update the data set
      dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData, 
                      dataName=modelName(model(curMVC)), from="")
      handleMessage(dfMessage)

      assign("oldMotionRow", "", controlEnv)
    }
    # add new highlighting
    if (all(rowName != testNewEmpty))
    {
      # then need to toggle old highlighting
      highIndex<-match(colName, colnames(virtualData))
      rowIndex<-match(rowName, rownames(virtualData))
      highValue<-virtualData[rowIndex, highIndex]
      # toggle the highlighting
      newHValue<-!highValue

      mData<-as.list(newHValue)
      names(mData)<-rowName

      # update the data set
      dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                      dataName=modelName(model(curMVC)), from="")
      handleMessage(dfMessage)

      assign("oldMotionRow", rowName, controlEnv)      
    }

    # now need to update the MVC
    # need to get the curMVC again because the model has changed!
    curMVC<-getMVC(mName)
    controller(curMVC)<-controlEnv

    MVCList<-get("MVCList", mvcEnv)

    # need to reassign the MVC
    allNames <- getModelNames(sort = FALSE)
    mvcIndex <- match(mName, allNames)
    MVCList[[mvcIndex]]<-curMVC
    assign("MVCList", MVCList, mvcEnv)
  }  
}

##########
# a callback function for coloring a point in a scatterplot
##########
colorPoint<-function(rowName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/30/06 added so that users can interact with views that don't belong to 
  # the active MVC
  curMVC<-getMVCFromWinNum()
  controlEnv<-controller(curMVC)

  # column name
  colName<-"color"

  # need to change the virtualData slot in model object 
  # using messages
  curColor<-get("currentColor", controlEnv)
  mData<-as.list(rep(curColor, length(rowName)))
  names(mData)<-rowName

  # if a point was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)

  # also want to change the pch so the color fills the point
  newmData<-as.list(rep(19, length(rowName)))
  names(newmData)<-rowName
  newdfMessage<-new("gUpdateDataMessage", type="pch", mData=newmData,
                    dataName=modelName(model(curMVC)), from="")
  handleMessage(newdfMessage)
}

##########
# added 6/7/05 
# a callback function to highlighting a point
##########
highlightPoint<-function(rowName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/30/06 added so that users can interact with views that don't belong to 
  # the active MVC
  curMVC<-getMVCFromWinNum()
  virtualData<-virtualData(model(curMVC))

  # column name
  colName<-"highlit"

  # need to change the virtualData slot in model object 
  # using messages
  # need to toggle the highlit value for this row
  # so need to get current value
  colIndex<-match(colName, colnames(virtualData))
  rowIndex<-match(rowName, rownames(virtualData))
  curHValue<-virtualData[rowIndex, colIndex]
  newHValue<-!curHValue

  mData<-as.list(newHValue)
  names(mData)<-rowName

  # if a point was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)
}

##########
# added 6/7/05 
# a callback function to hide a point
##########
hidePoint<-function(rowName)
{
#  activeMVC<-get("activeMVC", mvcEnv)
#  curMVC<-getMVC(activeMVC)
  # 5/30/06 added so that users can interact with views that don't belong to 
  # the active MVC
  curMVC<-getMVCFromWinNum()
  virtualData<-virtualData(model(curMVC))

  # column name
  colName<-"hide"

  # need to change the virtualData slot in model object 
  # using messages
  # need to toggle the hide value for this row
  # so need to get current value
  colIndex<-match(colName, colnames(virtualData))
  rowIndex<-match(rowName, rownames(virtualData))
  curHValue<-virtualData[rowIndex, colIndex]
  newHValue<-!curHValue

  mData<-as.list(newHValue)
  names(mData)<-rowName

  # if a point was clicked, then we want to create a 
  # gUpdateDataMessage
  dfMessage<-new("gUpdateDataMessage", type=colName, mData=mData,
                 dataName=modelName(model(curMVC)), from="")
  handleMessage(dfMessage)
}

##########
# assign a callback function to an event
# will be called whenever the user wants to set a function to a mouse over or
# a button click event
# eventList is a character string of the event name that has a 
#  list in controlEnv
##########
setCallFunEvent<-function(eventList)
{
  # first make sure there are MVCs
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)

    w<-get("controlWin", mvcEnv)

    # reset values before opening new view
    if (length(get("frMVC", mvcEnv)) > 0)
      setEmptyView()  
    else
      # make sure old action gets set when the first window is loaded
      assign("oldAction", get("curAction", mvcEnv), mvcEnv)    

    # 5 rows and 6 columns
    tab<-gtkTableNew(5, 6, TRUE)
    vbox<-get("vBox", mvcEnv)
    frMVC<-get("frMVC", mvcEnv)

    # remove the frame from the control window
    if (length(frMVC) > 0)
    {
      gtkContainerRemove(vbox, frMVC)

      # reset the frame to empty
      assign("frMVC", c(), mvcEnv)
    }

    frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
    frMVC$Add(tab)

    gtkBoxPackStart(vbox, frMVC)
    assign("frMVC", frMVC, mvcEnv)

    frAvail<-gtkFrame(paste("Available Functions for", eventList))
    frCurrent<-gtkFrame(paste("Current Function set for", eventList))
    gtkTableAttach(tab, frAvail, 0, 3, 0, 4, xpadding=5, ypadding=5)
    gtkTableAttach(tab, frCurrent, 3, 6, 0, 4, xpadding=5, ypadding=5) 

    # now need to populate these frames
    availFunctions<-get(eventList, controlEnv)
    shortDesc<-unlist(lapply(availFunctions, function(y)
                      {shortName(y$eventFun)})) 

    # need to show these short descriptions in frAvail   
    # make a vector of radio buttons
    if (length(shortDesc) > 0)
    {
      # may possibly need this to be scrolled
      vbox<-gtkVBox()
      # make a vector of radio buttons   
      radButs<-list()
      radBut<-gtkRadioButtonNewWithLabelFromWidget(label=shortDesc[1])
      radButs[[1]]<-radBut
      gtkBoxPackStart(vbox, radBut)
      if (length(shortDesc) > 1)
        for (i in 2:length(shortDesc))
        {
          newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                            label=shortDesc[i])
          gtkBoxPackStart(vbox, newradBut)
          radButs[[i]]<-newradBut
        } 
      frAvail$Add(vbox)
 
      # then add one more radio button for no function
      newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                            label="no action")
      gtkBoxPackStart(vbox, newradBut)
      radButs[[length(shortDesc)+1]]<-newradBut
    }
    # find out if any function is assigned
    activeFun<-unlist(lapply(availFunctions, function(y)
                            {y$assigned==TRUE}))        
    if (any(activeFun))
    {
      curActiveIndex<-which(activeFun)
      if (length(curActiveIndex) == 1)
      {
        activeLab<-gtkLabel(shortName(availFunctions[[curActiveIndex]][["eventFun"]]))
      }
    }
    else
      activeLab<-gtkLabel(paste("There is currently no action\n set to", 
                              eventList))
    frCurrent$Add(activeLab)

    setBut<-gtkButton("Set")
    gtkTableAttach(tab, setBut, 2, 4, 4, 5, xpadding=10, ypadding=5)

    # need a callback function for the set button
    gtkAddCallback(setBut, "clicked",
      function(obj, ev) 
      {
        # if there are any spreadsheets in activeMVC remove selection
        removeSpreadSelect()
        # find out which button is active
        if (length(shortDesc) > 0)      
        {
          for (i in 1:length(radButs))
          {
            if (gtkObjectGetArgs(radButs[[i]], "active") == TRUE)
              curIndex<-i
          }
          # then no action was chosen
          if (curIndex > length(shortDesc))
          {
            # need to make sure that assigned is set to FALSE for all functions
            if (any(activeFun))
            {
              availFunctions[[curActiveIndex]][["assigned"]]<-FALSE
              assign(eventList, availFunctions, controlEnv)
              # now recall this function to reset the view
              setCallFunEvent(eventList)
            }
          }
          # then one of the functions was chosen
          else
          {
            # first set the old function assigned to FALSE
            if (any(activeFun))
              availFunctions[[curActiveIndex]][["assigned"]]<-FALSE
            # now set the new function assigned to TRUE
            availFunctions[[curIndex]][["assigned"]]<-TRUE
            assign(eventList, availFunctions, controlEnv)
 
            # need to call the preprocessing function
            preprocess<-preprocessFun(availFunctions[[curIndex]][["eventFun"]])
            if (length(preprocess) > 0)
              for (i in 1:length(preprocess))
              {
                do.call(preprocess[[i]], list())
              }

            # now recall this function to reset the view
            setCallFunEvent(eventList)
          }
        }
        return(TRUE)
      }
    )
  }
  else
    if (length(get("controlWin", mvcEnv)) > 0)
      setEmptyView()  
}

##########
# added 6/8/05
# when a new callback function is linked to an event want to remove
# any selections on a spreadsheet view
##########
removeSpreadSelect<-function()
{
  activeMVC<-get("activeMVC", mvcEnv)
  curMVC<-getMVC(activeMVC)

  viewList<-viewList(curMVC)
  controlEnv<-controller(curMVC)

  if (length(viewList) > 0)
  {
    booSpread<-any(unlist(lapply(viewList, function(x) {is(x,
                                           "spreadView")})))
    if (booSpread)
    {
      # don't want to cause a click event to occur for the spreadsheet
      # so set contLoop variable to TRUE before unselecting rows
      assign("contLoop", TRUE, controlEnv)
      controller(curMVC)<-controlEnv
      MVCList<-get("MVCList", mvcEnv)
      allNames <- getModelNames(sort = FALSE)
      index <- match(activeMVC, allNames)
      MVCList[[index]]<-curMVC
      assign("MVCList", MVCList, mvcEnv)
      
      spreadIndex<-which(unlist(lapply(viewList, function(x) {is(x, 
                                           "spreadView")})))
      curSpread<-viewList[[spreadIndex]]
      curCList<-clist(curSpread)
      gtkCListUnselectAll(curCList)

      # now need to reset contLoop variable to FALSE
      curMVC<-getMVC(activeMVC)
      curVList<-viewList(curMVC)
      controlEnv<-controller(curMVC)

      assign("contLoop", FALSE, controlEnv)
      controller(curMVC)<-controlEnv
      MVCList<-get("MVCList", mvcEnv)
      allNames <- getModelNames(sort = FALSE)
      index <- match(activeMVC, allNames)
      MVCList[[index]]<-curMVC
      assign("MVCList", MVCList, mvcEnv)
    }
  }
}

