# added 10/24/05
mvcEnv<-new.env(parent=.GlobalEnv)

###########
# added 10/24/05 to set variables in mvcEnv
# will be used when using the MVC class definition
###########
setGlobalEnvVariables<-function()
{
  # controlWin is the gtkWindow
  assign("controlWin", c(), mvcEnv)
  # vBox is the vertical box that contains the main menu and frame, frMVC
  assign("vBox", c(), mvcEnv)
  # frMVC will hold the table
  assign("frMVC", c(), mvcEnv)  
  # for adding menu items to the main menu
  # ag is the accelerator group
  assign("ag", c(), mvcEnv)

  # the current action for the control window view
  assign("curAction", "", mvcEnv)
  # this is the last action that changed the control window appearance
  assign("oldAction", "", mvcEnv)

  # the default control window dimensions
  assign("winDefaultHeight", 500, mvcEnv)
  assign("winDefaultWidth", 300, mvcEnv)

  # menuBar is the main menu bar (all menus are added to the menu bar)
  assign("menuBar", c(), mvcEnv)
  # menus is the list of menus (such as file, edit, display, ...)
  assign("menus", list(), mvcEnv)

  # to store the window menu item - needed to update the menu when
  # sub menu items are added or removed from the window menu
  assign("windowItem", c(), mvcEnv)
  # to count the number of windows that have been opened
  assign("winCounter", 0, mvcEnv)
  # these are the global key values that all windows will share
  # need to store the key values for the menu item accelerators
  assign("keyVals", list(), mvcEnv)

  # the possible events that the user can match a callback function to;
  # each event will have its own list in controlEnv that contains
  # all the gEventFun objects that can be linked to this event
  assign("possEvents", c("mouseOver", "leftButtonClick", 
    "middleButtonClick", "rightButtonClick"), mvcEnv)

  # store the model name of the active MVC
  assign("activeMVC", "", mvcEnv)
  # store the active view's winNum - so when a view is interacted with, it can
  # be determined which MVC to update
  assign("activeWinNum", c(), mvcEnv)
  # the list of created MVCs
  assign("MVCList", list(), mvcEnv)
  
  assign("defaultPlotData", list(color="black", pch=1, highlit=FALSE, 
        hide=FALSE), mvcEnv)

  # for when we want to fill in a point
  assign("fillPch", 19, mvcEnv)

  # store the color browser window if it is opened
  assign("colorBrowser", c(), mvcEnv)

  # for determining an available data frame to load
  assign("availDF", "", mvcEnv)

  # 12/2/05 this list will include new callback functions that the user
  # has added from the command line (the function, createDefaultCallEvents, 
  # will check this variable when a new MVC object is loaded)
  assign("newCallFuns", list(), mvcEnv)

  # want to reassign these variables as well
  createMenuItemVariables()
}

############
# added 10/24/05 to set the controller environment default variables for
# each new MVC object
############
setControllerDefaults<-function(newMVC)
{
  controlEnv<-controller(newMVC)

  assign("currentColor", "black", controlEnv)

  # to make a window modal
  assign("notDone", TRUE, controlEnv)

  # need to store the key values for the menu item accelerators
  assign("localKeyVals", list(), controlEnv)

  # either the name of the new file or the contents of the file
  assign("loadedFile", NULL, controlEnv)
  # the name of the new loaded file
  assign("curName", "", controlEnv)

  # for the old node name that was shown in a tooltip
  assign("oldTTName", "", controlEnv)
  # for the old tooltip gtk window
  assign("oldTTWin", c(), controlEnv)
  # for the named vector of tooltips
  assign("ttToShow", c(), controlEnv)

  # store this with each MVC because each MVC may want a different response
  # to events!!
  # a list for each event that can link to gEventFun objects
  # each list element should be a list with 2 elements
  # the first element is the gEventFun object and the second
  # element is whether this function is currently linked to the event
  assign("mouseOver", list(), controlEnv)
  assign("leftButtonClick", list(), controlEnv)
  assign("middleButtonClick", list(), controlEnv)
  assign("rightButtonClick", list(), controlEnv)
  # to record which events the user wants to add the function to
  assign("selEvents", c(), controlEnv)

  # need an active plot window as well as an active device
  # this is needed for tooltips so that you don't end up with a 
  # open tooltip window that should have been closed
  assign("activePlotWindow", c(), controlEnv)

  # this is stored for motion notify event
  assign("oldMotionRow", "", controlEnv)
  # need this variable when selecting a row in spreadsheet so it doesn't
  # end up in an infinite loop
  assign("contLoop", FALSE, controlEnv)

  # to assign which variables are X and Y
  assign("chosenX", "", controlEnv)
  assign("chosenY", "", controlEnv)

  # used when searching for a point by row name
  assign("findInfo", "", controlEnv)
  assign("findOn", "", controlEnv)

  # to choose X/Y variables for plotting
  assign("toggleButtonXList", list(), controlEnv)
  assign("toggleButtonYList", list(), controlEnv)
}

#############
# changed 10/24/05 to account for mvcEnv
# createControlWindow creates a gtk window that controls
# what happens when interacting with a plot or when linking
# plots
#############
createControlWindow<-function()
{
  # check to ensure that a control window is not already open
  if (length(ls(env=mvcEnv)) > 0)
  {
    oldWin<-get("controlWin", mvcEnv)
  }
  else
  {
    # this function will be called in the .onLoad function
    setGlobalEnvVariables()
    oldWin<-get("controlWin", mvcEnv)
  }

  if (length(oldWin)==0)
  {
    w<-gtkWindow(show=FALSE)

    # the vertical box will never be removed from the window, but
    # the table will be removed depending on which menu item is
    # chosen (table will be added later)
    vBox<-gtkVBoxNew()
    frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
    assign("vBox", vBox, mvcEnv)
    assign("frMVC", frMVC, mvcEnv)
    assign("controlWin", w, mvcEnv)

    menuBar<-createMainMenu()

    # need to create boxes or frames
    gtkBoxPackStart(vBox, menuBar, expand=FALSE, fill=FALSE)
    gtkBoxPackStart(vBox, frMVC, expand=TRUE, fill=TRUE)

    gtkContainerAdd(w, vBox)

    gtkObjectSetArgs(w, "allow_shrink"=TRUE)
    gtkObjectSetArgs(w, "allow_grow"=TRUE)
    gtkObjectSetArgs(w, "auto_shrink"=TRUE)
    gtkContainerSetResizeMode(w, GtkResizeMode[1])

    w$SetUsize(get("winDefaultHeight", mvcEnv),
               get("winDefaultWidth", mvcEnv))

    w$Show()

    # need a delete event callback in case the user closes
    # the window without using the Quit menu item
    gtkAddCallback(w, "delete_event",
      function(obj,ev)
      {
        setGlobalEnvVariables()
        w$Destroy()
        return(TRUE)
      }
    )
  }

  invisible(NULL)
}

#############
# createMainMenu creates a top level menu for
# the control window
#
# note: don't need to pass the window because it can
# be accessed through the environment variable, controlWin
#
# added a new way to set up the menus using the controlEnv variables
# that are created in the function, createMenuItemVariables
#############
createMainMenu<-function()
{
  # list of menus to store in mvcEnv
  menus<-list()

  ag<-gtkAccelGroup()
  win<-get("controlWin", mvcEnv)
  gtkAccelGroupAttach(ag, win)
  assign("ag", ag, mvcEnv)

  #following the code in gtk.org Menu example
  # want to see if callbacks work
  fileMenu<-gtkMenuNew()
  editMenu<-gtkMenuNew()
  displayMenu<-gtkMenuNew()
  eventMenu<-gtkMenuNew()

  # added the control part to the label because it wasn't automatically
  # doing it using the GtkAccelFlags like it's supposed to (I think)
  fileItems<-get("fileItems", mvcEnv)
  for (i in 1:length(fileItems))
  {
    newItem<-createMenuItem(fileItems[[i]]$text, ag, "activate",
             GdkModifierType[3], GtkAccelFlags[2], fileItems[[i]]$action)
    gtkMenuAppend(fileMenu, newItem)
  }
  
  editItems<-get("editItems", mvcEnv)
  for (i in 1:length(editItems))
  {
    newItem<-createMenuItem(editItems[[i]]$text, ag, "activate",
             GdkModifierType[3], GtkAccelFlags[2], editItems[[i]]$action)
    gtkMenuAppend(editMenu, newItem)
  }

  displayItems<-get("displayItems", mvcEnv)
  for (i in 1:length(displayItems))
  {
    newItem<-createMenuItem(displayItems[[i]]$text, ag, "activate",
             GdkModifierType[3], GtkAccelFlags[2], displayItems[[i]]$action)
    gtkMenuAppend(displayMenu, newItem)
  }

  eventItems<-get("eventItems", mvcEnv)
  for (i in 1:length(eventItems))
  {
    newItem<-createMenuItem(eventItems[[i]]$text, ag, "activate",
             GdkModifierType[3], GtkAccelFlags[2], eventItems[[i]]$action)
    gtkMenuAppend(eventMenu, newItem)
  }
  
  menuBar<-gtkMenuBarNew()
  gtkWidgetShow(menuBar)

  # GdkModifierType[4] is alt
  fileItem<-createMenuItem("_File", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])

  editItem<-createMenuItem("_Edit", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])

  displayItem<-createMenuItem("_Display", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])

  eventItem<-createMenuItem("E_vent", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])

  windowItem<-createMenuItem("_Window", ag, "activate-item",
                           GdkModifierType[4], GtkAccelFlags[2])

  assign("windowItem", windowItem, mvcEnv)

  gtkMenuItemSetSubmenu(fileItem, fileMenu)
  gtkMenuItemSetSubmenu(editItem, editMenu)
  gtkMenuItemSetSubmenu(displayItem, displayMenu)
  gtkMenuItemSetSubmenu(eventItem, eventMenu)

  gtkMenuBarAppend(menuBar, fileItem)
  gtkMenuBarAppend(menuBar, editItem)
  gtkMenuBarAppend(menuBar, displayItem)
  gtkMenuBarAppend(menuBar, eventItem)
  gtkMenuBarAppend(menuBar, windowItem)

  menus[["file"]]<-fileMenu
  menus[["edit"]]<-editMenu
  menus[["display"]]<-displayMenu
  menus[["event"]]<-eventMenu
  assign("menus", menus, mvcEnv)

  assign("menuBar", menuBar, mvcEnv)

  addWindowMenu()

  return(menuBar)
}

#####
# addWindowMenu is used after calling gtkMenuItemRemoveSubmenu(windowItem)
# to reset the window menu
#####
addWindowMenu<-function()
{
  windowMenu<-gtkMenuNew()

  windowItem<-get("windowItem", mvcEnv)

  menus<-get("menus", mvcEnv)
  menus[["window"]]<-windowMenu
  assign("menus", menus, mvcEnv)

  controlWinItem<-addSubMenuItem("window",  "Control _Window  Ctrl+W",
                                 "activateWindow", winNum=0)

  # need to add to the window menu for the plots
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    allViews<-lapply(MVCList, viewList)
    for (i in 1:length(allViews))
    {
      curViewList<-allViews[[i]]
      if (length(curViewList) > 0)
      {
        allWinNums<-unlist(lapply(curViewList, winNum))
        for (j in 1:length(allWinNums))
        {
          curLabel<-paste("Window _", allWinNums[j],
                    "  Ctrl+", allWinNums[j], sep="")
          newMenuItem<-addSubMenuItem("window", curLabel, "activateWindow",
                   winNum=allWinNums[j])         
        }
      }
    }
  }

  gtkMenuItemSetSubmenu(windowItem, windowMenu)

  menus<-get("menus", mvcEnv)
  menus[["window"]]<-windowMenu
  assign("menus", menus, mvcEnv)
}

############
# create one menu item with an accelerator and possibly, a callback
############
createMenuItem<-function(labelText, ag, method, modType,
                         accelFlag, curAction="", ...)
{
  newItem<-gtkMenuItemNew()
  newLabel<-gtkAccelLabelNew(labelText)
  newKey<-gtkLabelParseUline(newLabel, labelText)

  gtkWidgetAddAccelerator(newItem, method, ag, newKey, modType,
                          accelFlag)
  gtkContainerAdd(newItem, newLabel)
  gtkWidgetShow(newItem)

  if (method=="activate")
  {
    gtkAddCallback(newItem, "activate",
      function(obj)
      {
        updateCurAction(curAction, ...)
      }
    )
  }

  addToKeyVals(labelText, curAction, ...)

  return(newItem)
}

#############
# addMenuItem adds a menu item to the main menu after the main menu
# has already been created
# meant to be used if the user wants to add a menu item from the
# command line
# ##FIXME: but it would be better if all code went through here, then
# ##you don't have one system for you and another for the users
#
#
# menuName is the name the menu will take in the list menus, which
# is stored in mvcEnv (so the menu can be accessed again)
#############
addMenuItem<-function(menuName, labelText, ag=get("ag", mvcEnv),
                      method="activate-item", modType=GdkModifierType[4],
                      accelFlag=GtkAccelFlags[2])
{
  ##check to see if it is there
  if( !is.null(mvcEnv$menus[[menuName]]) )
      stop(paste(menuName, "is already in the menu"))

  newItem<-createMenuItem(labelText, ag, method, modType, accelFlag)
  menuBar<-get("menuBar", mvcEnv)
  gtkMenuBarAppend(menuBar, newItem)
  gtkWidgetShow(menuBar)

  # make a sub menu for this new menu item (so things can be added to
  # it later)
  newMenu<-gtkMenuNew()
  gtkMenuItemSetSubmenu(newItem, newMenu)

  # need to also add the new menu item to menus list
  mvcEnv$menus[[menuName]] = newMenu
}

getItemLabel = function(x) {
    if( !inherits(x, "GtkMenuItem") )
        stop("must have a GtkMenuItem")
    gtkLabelGet(gtkContainerGetChildren(x)[[1]])
}

getMenuItemLabels = function(x) {
    if( !inherits(x, "GtkMenu") )
        stop("must have a GtkMenu")
    v1 = gtkContainerGetChildren(x)
    sapply( v1, getItemLabel)
}

##FIXME: needs to be hidden by a namespace
.hasItem = function(menu, itemName) {
    if( !inherits(menu, "GtkMenu") || !is.character(itemName) ||
    length(itemName) != 1)
        stop("wrong argument")
    itemLabs = getMenuItemLabels(menu)
    if( length(itemLabs)==0 )
        return(numeric(0))
    grep(paste("^", itemName, sep=""), itemLabs)
}

##############
# addSubMenuItem is to add a sub menu item to an already existing menu
# first add the menu item using addMenuItem and then add sub menu items
# using this function (add the menu item if you don't want to use file,
# edit, display or viewMode)
#
# menuName is the name of the menu that this sub menu item will be added to
# this must match the name of one of the list items in menus in mvcEnv
# curAction should be the function to call when this item is activated
# note that this function cannot have any parameters (see updateControlWindow)
##############
addSubMenuItem<-function(menuName, labelText, action,
                       ag=get("ag", mvcEnv), method="activate",
                       modType=GdkModifierType[3], accelFlag=GtkAccelFlags[2],
                       ...)
{
  # should check that the accelerator is not already in use!!
  ##first check to see if labelText is already there

  curMenu<- mvcEnv$menus[[menuName]]
  if( is.null(curMenu) )
      stop(paste(menuName, "is not a menu item"))

  hasLabel = .hasItem(curMenu, labelText)
  if( length(hasLabel) > 0 )
      stop("item", hasLabel, "has the same label")

  newItem<-createMenuItem(labelText=labelText, ag=ag, method=method,
           modType=modType, accelFlag=accelFlag, curAction=action, ...)

  gtkMenuAppend(curMenu,newItem)
  return(invisible(newItem))
}

###########
# update the current action environment variable and call update control window
# to reflect the change in current action
###########
updateCurAction<-function(curAct, ...)
{
  assign("curAction", curAct, mvcEnv)
  updateControlWindow(...)
}

##############
# updateControlWindow updates the control window
# after a different menu item was selected
##############
updateControlWindow<-function(...)
{
  curAction<-get("curAction", mvcEnv)

  # decide how to update based on curAction
  # problem it just keeps adding to the window (unless use table)

  # note that the default is to think that curAction is a function that
  # has no parameters (changed from setEmptyView)
  switch(curAction,
    "openf"=setLoadView(),
    "opend"=setOpenDView(),
    "plotD"=setPlotDView(),
    "viewData"=setViewDataView(),
    "quit"=askToQuit(),
    "replot"=askToReplot(),
    "reset"=askToReset(),
    "find"=askToFind(),
    "empty"=setEmptyView(),
    "setMouseOver"=setCallFunEvent("mouseOver"), 
    "setLeftBC"=setCallFunEvent("leftButtonClick"),
    "setMiddleBC"=setCallFunEvent("middleButtonClick"),
    "setRightBC"=setCallFunEvent("rightButtonClick"),
    "setActiveMVCGUI"=setActiveMVCGUI(),
    "deleteMVCGUI"=deleteMVCGUI(),
    do.call(curAction,list(...))
  )
}

#########
# ask the user which point they want to find
#########
askToFind<-function()
{
  booFind<-FALSE
  if (getNumberofViews() > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)

    # find if there is anything attached to the left button click event
    lbc<-get("leftButtonClick", controlEnv)
    # see if any of the functions are active
    activeLBCFun<-unlist(lapply(lbc, function(y)
        {y$assigned==TRUE}))
    curIndex<-which(activeLBCFun)
    if (length(curIndex)==1)
    {
      booFind<-TRUE
      curEventFun<-lbc[[curIndex]][["eventFun"]]
      actualFun<-callFun(curEventFun)

      curDF<-modelData(model(curMVC))
      rowNames<-rownames(curDF)
      actualData<-curDF

      # need to check that the user entered a valid row name
      getFindInfo()
       
      curText<-get("findInfo", controlEnv)
      findOn<-get("findOn", controlEnv)

      if (findOn=="Row" && curText %in% rowNames)
      {
        do.call(actualFun, list(curText))
      }

      if (findOn=="Data Element")
      {
        # check if the curText is a data element
        matchVals<-apply(actualData, 2, function(x)
           {
             # to get rid of leading white spaces
             x <- gsub("^[[:space:]]+", "", x)
             # to get rid of tailing white spaces
             x <- gsub("[[:space:]]+$", "", x)
             match(curText, x)
            })
        if (!all(is.na(matchVals)))
        {
          minMatch<-min(matchVals, na.rm=TRUE)
          curRow<-rowNames[minMatch]

          do.call(actualFun, list(curRow))
        }
      }
    } 
  }

  # let the user know that a plot must be open
  if (!booFind)
  {
    w<-gtkWindow(show=FALSE)
    lbl<-gtkLabel("There is no row to find because there \nare no views or the
  left button click event is not set.")
    but<-gtkButton("Ok")
    vbox<-gtkVBoxNew()
    hbox<-gtkHBoxNew()
    w$Add(vbox)
    gtkBoxPackStart(vbox, lbl)
    gtkBoxPackStart(vbox, hbox)
    gtkBoxPackStart(hbox, but, padding=60)
    w$Show()
    gtkAddCallback(but, "clicked",
      function(obj)
      {
        w$Destroy()
      }
    )
  }
}

#########
# open a window to get the string (row name) to find
#########
getFindInfo<-function()
{
  controlEnv<-controller(getMVC(get("activeMVC", mvcEnv)))

  assign("notDone", TRUE, controlEnv)
  win<-gtkWindow(show=FALSE)
  win$SetUsize(250, 100)
  gtkWindowSetModal(win, TRUE)

  tab<-gtkTableNew(3, 5, TRUE)
  vbox<-gtkVBoxNew()

  gtkBoxPackStart(vbox, tab)

  lbl<-gtkLabel("Find based on: ")

  gtkTableAttach(tab, lbl, 0, 2, 0, 1, xpadding=1, ypadding=5)
  comb<-gtkCombo()
  gtkComboSetPopdownStrings(comb,c("Row", "Data Element"))
  gtkComboSetValueInList(comb, TRUE, FALSE)
  gtkTableAttach(tab, comb, 2, 5, 0, 1, xpadding=1, ypadding=5)

  lblVal<-gtkLabel("Value: ")
  gtkTableAttach(tab, lblVal, 0, 2, 1, 2, xpadding=1, ypadding=5)
  ent<-gtkEntry()
  gtkTableAttach(tab, ent, 2, 5, 1, 2, xpadding=1, ypadding=5)
  okBut<-gtkButton("Ok")
  okBut$SetUsize(50, 20)
  gtkTableAttach(tab, okBut, 2, 4, 2, 3, xpadding=1, ypadding=5)
  win$Add(vbox)

  gtkAddCallback(okBut, "clicked",
    function(obj)
    {
      findOn<-gtkEntryGetText(gtkContainerGetChildren(comb)[[1]])
      curText<-gtkEntryGetText(ent)
      assign("findInfo", curText, controlEnv)
      assign("findOn", findOn, controlEnv)
      assign("notDone", FALSE, controlEnv)
      win$Destroy()
    }
  )

  win$Show()

  # need to make sure that notDone is set to FALSE if they user
  # closes the window without using the Ok button
  gtkAddCallback(win, "delete_event",
    function(obj, ev)
    {
      assign("notDone", FALSE, controlEnv)
    }
  )

  # to make the get find info window modal
  while (get("notDone", controlEnv))
    Sys.sleep(0.5)
}

##########
# open a window to check that the user wants to reset the dataframe
##########
askToReset<-function()
{
  if (getNumberofViews() > 0)
  {
    win<-gtkWindow(show=FALSE)
    win$SetUsize(300, 60)
    vBox<-gtkVBoxNew()
    lab<-gtkLabel("Do you want to reset the data to its original state?")
    hBox<-gtkHBoxNew()
    gtkBoxPackStart(vBox, lab, padding=5)
    gtkBoxPackStart(vBox, hBox)
    yesBut<-gtkButton("Yes")
    yesBut$SetUsize(40, 15)
    noBut<-gtkButton("No")
    noBut$SetUsize(40, 15)
    gtkBoxPackEnd(hBox, yesBut, expand=FALSE, fill=FALSE, padding=5)
    gtkBoxPackEnd(hBox, noBut, expand=FALSE, fill=FALSE, padding=5)
    win$Add(vBox)
    gtkAddCallback(yesBut, "clicked",
      function(obj)
      {
        activeMVC<-get("activeMVC", mvcEnv)
        # need to create an update data message
        # type="reset"
        dfMessage<-new("gUpdateDataMessage", type="reset", mData=list(),
                       dataName=activeMVC, from="")
        handleMessage(dfMessage)

        win$Destroy()
      }
    )
    gtkAddCallback(noBut, "clicked",
      function(obj)
      {
        win$Destroy()
      }
    )
    win$Show()
  }

  # let the user know that no resetting was done
  else
  {
    w<-gtkWindow(show=FALSE)
    lbl<-gtkLabel("There is no data to reset \nbecause there are no plots.")
    but<-gtkButton("Ok")
    vbox<-gtkVBoxNew()
    hbox<-gtkHBoxNew()
    w$Add(vbox)
    gtkBoxPackStart(vbox, lbl)
    gtkBoxPackStart(vbox, hbox)
    gtkBoxPackStart(hbox, but, padding=60)
    w$Show()
    gtkAddCallback(but, "clicked",
      function(obj)
      {
        w$Destroy()
      }
    )
  }
}

##########
# open a window to check that the user wants to replot the dataframe
##########
askToReplot<-function()
{
  if (getNumberofViews() > 0)
  {
    win<-gtkWindow(show=FALSE)
    win$SetUsize(250, 60)
    vBox<-gtkVBoxNew()
    lab<-gtkLabel("Do you want to replot the data?")
    hBox<-gtkHBoxNew()
    gtkBoxPackStart(vBox, lab, padding=5)
    gtkBoxPackStart(vBox, hBox)
    yesBut<-gtkButton("Yes")
    yesBut$SetUsize(40, 15)
    noBut<-gtkButton("No")
    noBut$SetUsize(40, 15)
    gtkBoxPackEnd(hBox, yesBut, expand=FALSE, fill=FALSE, padding=5)
    gtkBoxPackEnd(hBox, noBut, expand=FALSE, fill=FALSE, padding=5)
    win$Add(vBox)
    gtkAddCallback(yesBut, "clicked",
      function(obj)
      {
        activeMVC<-get("activeMVC", mvcEnv)
        # need to replot all these views!
        curviewList<-viewList(getMVC(activeMVC))

        if (length(curviewList) > 0)
        {
          uvMessage<-new("gUpdateViewMessage", type="redrawView",
                         mData=list(), dataName=activeMVC)
          handleMessage(uvMessage)
        }

        win$Destroy()
      }
    )
    gtkAddCallback(noBut, "clicked",
      function(obj)
      {
        win$Destroy()
      }
    )
    win$Show()
  }

  # let the user know that there was no plot to replot
  else
  {
    w<-gtkWindow(show=FALSE)
    lbl<-gtkLabel("There is no view to replot.")
    but<-gtkButton("Ok")
    vbox<-gtkVBoxNew()
    hbox<-gtkHBoxNew()
    w$Add(vbox)
    gtkBoxPackStart(vbox, lbl)
    gtkBoxPackStart(vbox, hbox)
    gtkBoxPackStart(hbox, but, padding=40)
    w$Show()
    gtkAddCallback(but, "clicked",
      function(obj)
      {
        w$Destroy()
      }
    )
  }
}

###############
# askToQuit opens a widget that asks the user
# if they really want to quit when the user has
# chosen quit on the main menu
#
# this function should close all windows if the
# user chooses yes!!!
###############
askToQuit<-function()
{
  win<-gtkWindow(show=FALSE)
  win$SetUsize(250, 60)
  vBox<-gtkVBoxNew()
  lab<-gtkLabel("Are you sure you want to quit?")
  hBox<-gtkHBoxNew()
  gtkBoxPackStart(vBox, lab, padding=5)
  gtkBoxPackStart(vBox, hBox)
  yesBut<-gtkButton("Yes")
  yesBut$SetUsize(40, 15)
  noBut<-gtkButton("No")
  noBut$SetUsize(40, 15)
  gtkBoxPackEnd(hBox, yesBut, expand=FALSE, fill=FALSE, padding=5)
  gtkBoxPackEnd(hBox, noBut, expand=FALSE, fill=FALSE, padding=5)
  win$Add(vBox)
  gtkAddCallback(yesBut, "clicked",
    function(obj)
    {
      # call a function to destroy all windows
      clearAllWindows()
      win$Destroy()
      return(TRUE)
    }
  )
  gtkAddCallback(noBut,"clicked",
    function(obj)
    {
      win$Destroy()
      return(TRUE)
    }
  )
  win$Show()
}

#############
# clearAllWindows destroys all windows stored in mvcEnv
#############
clearAllWindows<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    for (i in 1:length(MVCList))
    {
      curViewList<-viewList(MVCList[[i]])
      if (length(curViewList) > 0)
        for (j in 1:length(curViewList))
        {
          setDeleteEvents(curViewList[[j]])
        }
    }
  }

  # need to remove the control window
  cWin<-get("controlWin", mvcEnv)
  if (length(cWin) > 0)
  {
    cWin$Destroy()
    # reset the control window to empty
    assign("controlWin", c(), mvcEnv)
  }

  setGlobalEnvVariables()
}

###########
# resetWinSize resets the gtkWindow to its default size
# (having trouble with this)
###########
resetWinSize<-function()
{
  w<-get("controlWin", mvcEnv)
  w$SetUsize(get("winDefaultHeight", mvcEnv),
             get("winDefaultWidth", mvcEnv))
}

#############
# setEmptyView sets the view on the control window to
# only have the main menu
#############
setEmptyView<-function()
{
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)

  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)

    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  # put in a new frame
  frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)

  # reset the chosen X and Y to unknown
  assign("chosenX", "", mvcEnv)
  assign("chosenY", "", mvcEnv)

  # since the control window view is being changed, we want
  # to store this action as oldAction;
  # only update old action if the action is one that changes the
  # control window - and any time the control window is being
  # changed, setEmptyView is called
  oldAction<-get("curAction", mvcEnv)
  assign("oldAction", oldAction, mvcEnv)
}

##############
# setPlotDView sets the control window so it
# shows the view of loaded dataframes to
# choose which one to plot
##############
setPlotDView<-function()
{
  # first make sure there are MVCs
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    w<-get("controlWin", mvcEnv)
    activeMVC<-get("activeMVC", mvcEnv)

    # reset values before opening new view
    if (length(get("frMVC", mvcEnv)) > 0)
      setEmptyView()
    else
      # make sure old action gets set when the first window is loaded
      assign("oldAction", get("curAction", mvcEnv), mvcEnv)

    tab<-gtkTableNew(8, 8, TRUE)
    vbox<-get("vBox", mvcEnv)
    frMVC<-get("frMVC", mvcEnv)

    # remove the frame from the control window
    if (length(frMVC) > 0)
    {
      gtkContainerRemove(vbox, frMVC)

      # reset the frame to empty
      assign("frMVC", c(), mvcEnv)
    }

    frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
    frMVC$Add(tab)

    gtkBoxPackStart(vbox, frMVC)
    assign("frMVC", frMVC, mvcEnv)

    # only allow users to plot the active MVC
    loadedDF<-activeMVC

    frVar<-gtkFrame("Data Variables")
    gtkTableAttach(tab, frVar, 2, 6, 0, 6, xpadding=10)
    setVariables(activeMVC, frVar)

    plotBut<-gtkButton("Plot")
    gtkTableAttach(tab, plotBut, 3, 5, 6, 8, xpadding=20, ypadding=20)

    gtkAddCallback(plotBut, "clicked",
      function(obj, ev)
      {
        dataF<-activeMVC
        curMVC<-getMVC(dataF)
        controlEnv<-controller(curMVC)

        # need to get the two plot variables and call plotting function
        chosenX<-get("chosenX", controlEnv)
        chosenY<-get("chosenY", controlEnv)

        if (chosenX != "" && chosenY != "")
        {
          # then need to call plotting function
          curDF<-getData(name=dataF)
          varNames<-names(curDF)
          numRows<-dim(curDF)[1]
          Xindex<-match(chosenX, varNames)
          Yindex<-match(chosenY, varNames)
          DFrows<-rownames(curDF)
          DFcolumns<-c(Xindex, Yindex)

          # create a message to create a view
          vMessage<-new("gAddViewMessage", dataName=dataF, type="sPlotView",
                  mData=list(dfRows=DFrows, xvar=chosenX, yvar=chosenY))
          handleMessage(vMessage)
        }
        return(TRUE)
      }
    )
  }
  else
    if (length(get("controlWin", mvcEnv)) > 0)
      setEmptyView()  
}

#############
# setViewDataView allows the user to view the data
#############
setViewDataView<-function()
{
  # first make sure there are MVCs
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)

    # reset values before opening new view
    if (length(get("frMVC", mvcEnv)) > 0)
      setEmptyView()
    else
      # make sure old action gets set when the first window is loaded
      assign("oldAction", get("curAction", mvcEnv), mvcEnv)

    tab<-gtkTableNew(4, 4, TRUE)
    vbox<-get("vBox", mvcEnv)
    frMVC<-get("frMVC", mvcEnv)

    # remove the frame from the control window
    if (length(frMVC) > 0)
    {
      gtkContainerRemove(vbox, frMVC)

      # reset the frame to empty
      assign("frMVC", c(), mvcEnv)
    }
    frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
    frMVC$Add(tab)

    gtkBoxPackStart(vbox, frMVC)
    assign("frMVC", frMVC, mvcEnv)

    # only allow users to create spreadsheet of active MVC
    loadedDF<-activeMVC

    butLabel<-paste("Show", activeMVC)
    showButton<-gtkButton(butLabel)
    gtkTableAttach(tab, showButton, 1, 3, 1, 3, ypadding=50, xpadding=30)

    gtkAddCallback(showButton, "clicked",
      function(obj)
      {
        showDF<-get("activeMVC", mvcEnv)

        if (showDF != "")
        {
          # check to see if the data is already shown in a spreadsheet
          curMVC<-getMVC(activeMVC)
          curviewList<-viewList(curMVC)
          booSpreadView<-unlist(lapply(curviewList, function(x) {is(x, 
                                 "spreadView")}))
          # if there are any spread views then this data set is already shown
          # in a spreadsheet
          if (!any(booSpreadView))
          {
            # create and handle a message to create a view
            vMessage<-new("gAddViewMessage", dataName=showDF, 
                          type="spreadView", mData=list())
            handleMessage(vMessage)
          }
        }
        return(TRUE)
      }
    )
  }  
  else
    if (length(get("controlWin", mvcEnv)) > 0)
      setEmptyView() 
}

#############
# changed 10/24/05 to account for mvcEnv
#
# setOpenDView allows the user to choose a data set to open
#############
setOpenDView<-function()
{
  w<-get("controlWin", mvcEnv)

  # reset values before opening new view
  if (length(get("frMVC", mvcEnv)) > 0)
    setEmptyView()
  else
    # make sure old action gets set when the first window is loaded
    assign("oldAction", get("curAction", mvcEnv), mvcEnv)

  tab<-gtkTableNew(8, 8, TRUE)
  vbox<-get("vBox", mvcEnv)
  frMVC<-get("frMVC", mvcEnv)

  # remove the frame from the control window
  if (length(frMVC) > 0)
  {
    gtkContainerRemove(vbox, frMVC)

    # reset the frame to empty
    assign("frMVC", c(), mvcEnv)
  }

  frMVC<-gtkFrameNew(label=get("activeMVC", mvcEnv))
  frMVC$Add(tab)

  gtkBoxPackStart(vbox, frMVC)
  assign("frMVC", frMVC, mvcEnv)

  # note that objects other than dataframes are returned from getData
  availDF<-getDataFromData()
  loadedDF<-getLoadedDF()
  # remove any dataframes from availDF that are already loaded
  if (!is.null(loadedDF))
    availDF<-availDF[!(availDF %in% loadedDF)]

  frAvail<-gtkFrame("Available Data")
  boxForAvail<-gtkVBox(TRUE, 2)
  sc<-gtkScrolledWindow()
  sc$SetPolicy("automatic", "automatic")
  availDfList<-gtkList()
  if (length(availDF) > 0)
    for (i in 1:length(availDF))
    {
      availDfListItem<-gtkListItemNewWithLabel(availDF[i])
      availDfList$Add(availDfListItem)
    }

  gtkListSetSelectionMode(availDfList, GtkSelectionMode[1])
  sc$AddWithViewport(availDfList)
  boxForAvail$PackStart(sc,expand=TRUE)
  frAvail$Add(boxForAvail)
  gtkTableAttach(tab, frAvail, 0, 3, 0, 8, ypadding=10)

  frLoaded<-gtkFrame("Loaded Data")
  boxForLoaded<-gtkVBox(TRUE, 2)
  scLoad<-gtkScrolledWindow()
  scLoad$SetPolicy("automatic", "automatic")
  loadedDfList<-gtkList()
  if (length(loadedDF) > 0)
    for (i in 1:length(loadedDF))
    {
      loadedDfListItem<-gtkListItemNewWithLabel(loadedDF[i])
      loadedDfList$Add(loadedDfListItem)
    }

  gtkListSetSelectionMode(loadedDfList, GtkSelectionMode[1])
  scLoad$AddWithViewport(loadedDfList)
  boxForLoaded$PackStart(scLoad, expand=TRUE)
  frLoaded$Add(boxForLoaded)
  gtkTableAttach(tab, frLoaded, 5, 8, 0, 8, ypadding=10)

  gtkAddCallback(availDfList, "select-child",
    function(obj, objchild)
    {
      # need to get the active element in the list
      selItem<-gtkChildren(objchild)[[1]][["label"]]
      # remove spaces from the text
      #selItem<-unlist(strsplit(selItem, " "))[1]
      assign("availDF", selItem, mvcEnv)
    }
  )

  loadBut<-gtkButton("Load")
  gtkTableAttach(tab, loadBut, 3, 5, 3, 5, xpadding=20, ypadding=20)

  gtkAddCallback(loadBut, "clicked",
    function(obj)
    {
      loadIfDF()
    }
  )
}

#############
# only load if a dataframe or matrix
#############
loadIfDF<-function()
{
  curDF<-get("availDF", mvcEnv)

  if (curDF != "")
  {

    # added for R-2.0.0
    # now need to break into 2 pieces if there are parentheses
    splitCurDf<-unlist(strsplit(curDF," "))

    if (length(splitCurDf) > 1)
    {
      # need to remove parentheses from second element
      removeParen<-splitCurDf[2]
      removeParen<-substr(removeParen, 2, nchar(removeParen)-1)
      dataParem<-removeParen
      dataName<-splitCurDf[1]
    }
    else
    {
      dataParem<-splitCurDf[1]
      dataName<-splitCurDf[1]
    }

    data(list=dataParem)

    # now check to make sure it has been loaded
    if (!is.na(match(dataName, ls(.GlobalEnv))))
    {
      # only load dataframes or matrices
      if (is.data.frame(eval(as.name(dataName))) ||
          is.matrix(eval(as.name(dataName))) )
      {
        dMessage<-new("gAddDataMessage",
                      data=list(data=eval(as.name(dataName))), 
                      dataName=dataName)
        handleMessage(dMessage)

        # refresh the view or add/remove the item
        setOpenDView()
      }
      else
      {
        # let the user know that the data is not a dataframe
        showNotDF(dataName)
      }
    }
    assign("availDF", "", mvcEnv)
  }
}

###############
# let the user know that the data was not a dataframe or matrix
###############
showNotDF<-function(curDF)
{
  cWin<-get("controlWin", mvcEnv)
  if (length(cWin) > 0)
  {
    win<-gtkWindow(show=FALSE)
    hbox<-gtkHBox()
    win$Add(hbox)
    lbl<-gtkLabel(paste(curDF,"is not a data frame or a \nmatrix so it could ",
                      "not be loaded."))
    gtkBoxPackStart(hbox, lbl, padding=5)
    okBut<-gtkButton("Ok")
    okBut$SetUsize(40, 20)
    gtkBoxPackStart(hbox, okBut, padding=5)

    gtkAddCallback(okBut, "clicked",
      function(obj)
      {
        win$Destroy()
      }
    )
    win$Show()
  }
  else
    print(paste(curDF, "is not a data frame or a matrix so it could",
                     "not be loaded."))
}

############
# getData returns everything from data()
############
getDataFromData<-function()
{
  curData<-data()
  curDF<-curData$results[,match("Item", dimnames(curData$results)[[2]])]
  scurDF<-sort(curDF)

  return(scurDF)
}

##########
# setVariables sets the dataframe variables in a list box
#
# dataF is the dataframe name, frVar is the frame that
# will hold the dataframe variables
##########
setVariables<-function(dataF, frVar)
{
  curMVC<-getMVC(dataF)
  controlEnv<-controller(curMVC)

  # need to reset the button boxes before adding anything
  resetVarButtonBoxes(frVar)

  # need to add the button boxes along with the variable labels
  varNames<-names(getData(name=dataF))

  # remove the variable names that pertain to default plot data
  defaultPlotNames<-names(get("defaultPlotData", mvcEnv))
  remNames<-(varNames %in% defaultPlotNames)
  varNames<-sort(varNames[!remNames])

  boxForVar<-gtkVBox(TRUE, 2)
  sc<-gtkScrolledWindow()
  sc$SetPolicy("automatic", "automatic")

  XbuttonList<-list()
  YbuttonList<-list()

  if (length(varNames) > 0)
  {
    # length(varNames)=# of rows, 3 columns, homogenous=FALSE??
    tab<-gtkTable(length(varNames), 3, FALSE)

    for (i in 1:length(varNames))
    {
      XbuttonList[[i]]<-gtkToggleButton("X")
      YbuttonList[[i]]<-gtkToggleButton("Y")
      lab<-gtkLabel(varNames[i])
      gtkTableAttach(tab, XbuttonList[[i]], 0, 1, i-1, i, xpadding=5)
      gtkTableAttach(tab, YbuttonList[[i]], 1, 2, i-1, i, xpadding=5)
      gtkTableAttach(tab, lab, 2, 3, i-1, i, xpadding=5)

# PROBLEM: when using this anonymous function the wrong index gets sent to
# setToggleX (the index is always the maximum value - the last index value) 
#      XbuttonList[[i]]$AddCallback("toggled",
#           function(btn)
#           {
#             print(gtkToggleButtonGetActive(btn))
##             setToggleX(i, dataF)
#             eval(substitute(setToggleX(j, dataFr), list(j=i, dataFr=dataF)))
#           })

      XbuttonList[[i]]$AddCallback("clicked",
        substitute(setToggleX(j, dataFr), list(j=i, dataFr=dataF)))
      YbuttonList[[i]]$AddCallback("clicked",
        substitute(setToggleY(j, dataFr), list(j=i, dataFr=dataF)))
    }
    sc$AddWithViewport(tab)
  }
  assign("toggleButtonXList", XbuttonList, controlEnv)
  assign("toggleButtonYList", YbuttonList, controlEnv)

  boxForVar$PackStart(sc, expand=FALSE)

  frVar$Add(boxForVar)
}

############
# setToggleX sets the chosen X value
#
# j is the index of the button that just changed state
# dataFr is the current dataframe
############
setToggleX<-function(j, dataFr)
{
#  print(j)
  setToggleInactive(j, "X", dataFr)
}

##############
# setToggleY sets the chosen Y value
#
# j is the index of the button that just changed state
# dataFr is the current dataframe
##############
setToggleY<-function(j, dataFr)
{
#  print(j)
  setToggleInactive(j, "Y", dataFr)
}

##############
# setToggleInactive sets the toggle buttons so
# only one button can be active at a time and it
# also sets the chosenX/chosenY values for plotting
#
# j is the index of the button that just changed state
# type is X or Y
# dataFr is the current dataframe
##############
setToggleInactive<-function(j, type, dataFr)
{
  curMVC<-getMVC(dataFr)
  controlEnv<-controller(curMVC)

  # need to check if there are two buttons active
  XbuttonList<-get("toggleButtonXList", controlEnv)
  YbuttonList<-get("toggleButtonYList", controlEnv)

  indexActive<-0
  numActive<-0
  for (i in 1:length(XbuttonList))
  {
    if (type=="X")
    {
      if (gtkToggleButtonGetActive(XbuttonList[[i]])==TRUE)
      {
        numActive<-numActive+1
        indexActive<-i
      }
    }
    else
    {
      if (gtkToggleButtonGetActive(YbuttonList[[i]])==TRUE)
      {
        numActive<-numActive+1
        indexActive<-i
      }
    }
  }

  if (numActive > 1)
    for (i in 1:length(XbuttonList))
    {
      if (i != j)
      {
        if (type=="X")
        {
          gtkToggleButtonSetActive(XbuttonList[[i]], FALSE)
        }
        else
        {
          gtkToggleButtonSetActive(YbuttonList[[i]], FALSE)
        }
      }
    }

  # if there is only one active toggle button - set that to be chosen
  else
  {
    # get the variable names
    varNames<-names(getData(name=dataFr))

    # remove the variable names that pertain to default plot data
    defaultPlotNames<-names(get("defaultPlotData", mvcEnv))
    remNames<-(varNames %in% defaultPlotNames)
    varNames<-sort(varNames[!remNames])

    if (type=="X")
      assign("chosenX", varNames[indexActive], controlEnv)
    else
      assign("chosenY", varNames[indexActive], controlEnv)
  }
}

###########
# resetVarButtonBoxes removes all children from the frVar
#
# frVar is the frame containing the button boxes for the
# dataframe variables
###########
resetVarButtonBoxes<-function(frVar)
{
  # remove all items from the frame
  kids<-gtkContainerGetChildren(frVar)
  if (length(kids) > 0)
    for (i in 1:length(kids))
    {
      gtkWidgetDestroy(kids[[i]])
    }
}

##############
# setLoadView opens a file explorer, which
# allows the user to load a file
# if the file loads a dataframe or a matrix, that data set
# is added to MVCList in mvcEnv
##############
setLoadView<-function()
{
  fileSel<-gtkFileSelection(title="Open a File", show=FALSE)
  # need to set the callbacks on these buttons
  okBut<-gtkFileSelectionGetOkButton(fileSel)
  canBut<-gtkFileSelectionGetCancelButton(fileSel)
  # need to hide 3 file buttons
  gtkFileSelectionHideFileopButtons(fileSel)

  fileSel$Show()

  gtkAddCallback(canBut, "clicked",
    function(obj)
    {
      fileSel$Destroy()
    }
  )
  gtkAddCallback(okBut, "clicked",
    function(obj)
    {
      fileEntry<-gtkFileSelectionGetFilename(fileSel)

      # need to determine the file extension, which will be the second
      # entry after splitting on "."
      fileSplitExt<-unlist(strsplit(fileEntry, "\\."))
      fileExt<-fileSplitExt[length(fileSplitExt)]

      fileSplitName<-unlist(strsplit(fileEntry, "\\/"))
      fileName<-fileSplitName[length(fileSplitName)]
      # need to remove the extension from fileName
      fileName<-unlist(strsplit(fileName, "\\."))[1]

      upFileExt<-toupper(fileExt)

      newData<-switch(upFileExt,
        "R"=source(file=fileEntry),
        "RDA"=load(file=fileEntry, envir=.GlobalEnv),
        "TAB"=read.table(file=fileEntry, header=TRUE),
        "CSV"=read.table(file=fileEntry, header=TRUE, sep=";"),
        "RDATA"=load(file=fileEntry, envir=.GlobalEnv),
        "TXT"=read.table(file=fileEntry, header=TRUE),
        showNotDF(fileName)
      )

      # now need to decide whether to load newData
      if (upFileExt == "R" || upFileExt == "RDA" || upFileExt == "TAB" ||
          upFileExt == "CSV" || upFileExt == "RDATA")
        loadDF(newData, upFileExt, fileName)

      fileSel$Destroy()

      # need to update the control window
      oldAction<-get("oldAction", mvcEnv)
      if (oldAction != "")
      {
        assign("curAction", get("oldAction", mvcEnv), mvcEnv)
        updateControlWindow()
      }
    }
  )
}

############
# loadDF checks whether the file just loaded was
# a dataframe - if it was a dataframe or matrix, it is
# added to MVCList in mvcEnv
############
loadDF<-function(newData, fileExt, fileName)
{
  if (fileExt=="R")
    actualData<-newData$value
  if (fileExt=="RDA" || fileExt=="RDATA")
    actualData<-eval(as.name(newData))
  if (fileExt=="TAB" || fileExt=="TXT" || fileExt=="CSV")
    actualData<-newData

  if (is.data.frame(actualData) || is.matrix(actualData))
  {
    dMessage<-new("gAddDataMessage", data=list(data=actualData), 
                  dataName=fileName)
    handleMessage(dMessage)
    print(paste(fileName, "has been loaded."))
  }
  else
  {
    showNotDF(fileName)
  }
}

##########
# getLoadedDF gets the names of the loaded dataframes in mvcEnv
##########
getLoadedDF<-function()
{
  curMVCs<-get("MVCList", mvcEnv)

  if (length(curMVCs) > 0)
  {
    curDFs<-unlist(lapply(curMVCs, function(x)
          {
            modelName(model(x))
          }
        ))
  }
  else
    curDFs<-c()
  return(curDFs)
}

###############
# createColorBrowser creates the color selection dialog so the user
# can pick a color
#
# note that the color browser is modal 
###############
createColorBrowser<-function()
{
  if (length(get("controlWin", mvcEnv)) > 0)
  {
    activeMVC<-get("activeMVC", mvcEnv)
    curMVC<-getMVC(activeMVC)
    controlEnv<-controller(curMVC)
  
    assign("notDone", TRUE, controlEnv)
    w<-gtkColorSelectionDialog(title="Pick a Color", show=FALSE)
    gtkWindowSetModal(w, TRUE)

    # w gives the whole window - we want to get certain elements of the window
    okBut<-gtkColorSelectionDialogGetOkButton(w)
    canBut<-gtkColorSelectionDialogGetCancelButton(w)
    helpBut<-gtkColorSelectionDialogGetHelpButton(w)
    colorSel<-gtkColorSelectionDialogGetColorsel(w)
    # hide the Help button
    gtkWidgetHide(helpBut)
    w$Show()

    # need to write callbacks for these buttons
    gtkAddCallback(okBut, "clicked",
      function(obj)
      {
        # need to set the color and destroy w
        curColor<-gtkColorSelectionGetColor(colorSel)
        curRed<-curColor[1]
        curGreen<-curColor[2]
        curBlue<-curColor[3]
        curColor<-rgb(curRed, curGreen, curBlue)
        # now assign the color to the environment
        assign("currentColor", curColor, controlEnv)
        assign("notDone", FALSE, controlEnv)
        w$Destroy()
        return(TRUE)
      }
    )
    gtkAddCallback(canBut, "clicked",
      function(obj)
      {
        assign("notDone", FALSE, controlEnv)
        w$Destroy()
        return(TRUE)
      }
    )

    # to make the color browser modal
    while (get("notDone", controlEnv))
      Sys.sleep(0.5)
  }
  else
    print(paste("Please call the function, setColor, to set the color. ",  
                "The default color is black."))
}

##########
# let the user delete a MVC object through the GUI - do not let the user
# delete the active MVC, though
##########
deleteMVCGUI<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 1)
  {
    w<-gtkWindow(show=FALSE)
    frMVC<-gtkFrame(label="Delete a MVC")
    w$Add(frMVC)
    allvbox<-gtkVBox()
    frMVC$Add(allvbox)

    modNames<-getModelNames()
    activeMVC<-get("activeMVC", mvcEnv)
    # need to remove the active MVC from the possibilities of MVCs to delete
    remModNames<-modNames[!(modNames %in% activeMVC)]

    sc<-gtkScrolledWindow()
    sc$SetPolicy("automatic", "automatic")

    MVCvbox<-gtkVBox()
    if (length(remModNames) > 0)
    {
      radButs<-list()
      radBut<-gtkRadioButtonNewWithLabelFromWidget(label=remModNames[1])
      gtkBoxPackStart(MVCvbox, radBut)
      radButs[[1]]<-radBut
      if (length(remModNames) > 1)
        for (i in 2:length(remModNames))
        {
          newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                            label=remModNames[i])
          gtkBoxPackStart(MVCvbox, newradBut)
          radButs[[i]]<-newradBut
        }     
    } 
    sc$AddWithViewport(MVCvbox)  
    gtkBoxPackStart(allvbox, sc)

    hbox<-gtkHBox()
    gtkBoxPackStart(allvbox, hbox)

    delBut<-gtkButton("Delete")
    canBut<-gtkButton("Cancel")

    gtkBoxPackStart(hbox, delBut)
    gtkBoxPackStart(hbox, canBut)

    w$Show()  

    # need a callback for the button press event
    gtkAddCallback(canBut, "clicked",
      function(obj, ev)
      {
        # destroy the window without doing anything
        w$Destroy()
        return(TRUE)
      }
    )
    gtkAddCallback(delBut, "clicked",
      function(obj, ev)
      {
        # first find out which name is checked and then delete that MVC
        if (length(remModNames) > 0)
        {
          for (i in 1:length(radButs))
          {
            if (gtkObjectGetArgs(radButs[[i]], "active") == TRUE)
              curIndex<-i
          }      
          deleteMVC<-remModNames[curIndex]

          # now need to remove the views for this MVC 
          delMVC<-getMVC(deleteMVC)
          vList<-viewList(delMVC)
        
          if (length(vList) > 0)
            for (i in 1:length(vList))
              setDeleteEvents(vList[[i]])          

          # then remove the MVC from MVCList in mvcEnv
          # find the index of the item to remove    
          allNames<-getModelNames(sort=FALSE)
          index<-match(deleteMVC, allNames)

          newMVCList<-MVCList[-index]
          assign("MVCList", newMVCList, mvcEnv)

          # set the control window to empty because the old action may be
          # for a different model type
          if (length(get("controlWin", mvcEnv)) > 0)
            setEmptyView()
        }

        # destroy the window
        w$Destroy()
        return(TRUE)
      }
    )
  }
  else
  {
    if (length(MVCList) == 0)
      print("There is no data loaded so there are no MVCs to delete.")
    else
      print(paste("The active MVC cannot be deleted", 
                  "and there is only one MVC loaded at this time.")) 
  }
}

###########
# setActiveMVCGUI creates the frame where the active MVC can be chosen from 
# all loaded MVCs
###########
setActiveMVCGUI<-function()
{
  MVCList<-get("MVCList", mvcEnv)
  if (length(MVCList) > 0)
  {
    w<-gtkWindow(show=FALSE)
    frMVC<-gtkFrame(label="Set Active MVC")
    w$Add(frMVC)
    allvbox<-gtkVBox()
    frMVC$Add(allvbox)

    modNames<-getModelNames()
    sc<-gtkScrolledWindow()
    sc$SetPolicy("automatic", "automatic")

    MVCvbox<-gtkVBox()
    if (length(modNames) > 0)
    {
      radButs<-list()
      radBut<-gtkRadioButtonNewWithLabelFromWidget(label=modNames[1])
      gtkBoxPackStart(MVCvbox, radBut)
      radButs[[1]]<-radBut
      if (length(modNames) > 1)
        for (i in 2:length(modNames))
        {
          newradBut<-gtkRadioButtonNewWithLabelFromWidget(radBut, 
                            label=modNames[i])
          gtkBoxPackStart(MVCvbox, newradBut)
          radButs[[i]]<-newradBut
        }     
    } 
    sc$AddWithViewport(MVCvbox)  
    gtkBoxPackStart(allvbox, sc)

    hbox<-gtkHBox()
    gtkBoxPackStart(allvbox, hbox)

    setBut<-gtkButton("Set")
    canBut<-gtkButton("Cancel")

    gtkBoxPackStart(hbox, setBut)
    gtkBoxPackStart(hbox, canBut)

    w$Show()  

    # need a callback for the button press event
    gtkAddCallback(canBut, "clicked",
      function(obj, ev)
      {
        # destroy the window without doing anything
        w$Destroy()
        return(TRUE)
      }
    )
    gtkAddCallback(setBut, "clicked",
      function(obj, ev)
      {
        # first find out which name is checked and then set the 
        # environment variable
        if (length(modNames) > 0)
        {
          for (i in 1:length(radButs))
          {
            if (gtkObjectGetArgs(radButs[[i]], "active") == TRUE)
              curIndex<-i
          }      
          activeMVC<-modNames[curIndex]
          assign("activeMVC", activeMVC, mvcEnv)

          # set the control window to empty because the old action may be
          # for a different model type
          if (length(get("controlWin", mvcEnv)) > 0)
            setEmptyView()
        }

        # destroy the window
        w$Destroy()
        return(TRUE)
      }
    )
  }
}
